@@ -1,7 +1,7 @@
 /*
  * ion/de/brush.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2005. 
+ * Copyright (c) Tuomo Valkonen 1999-2006. 
  *
  * Ion is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by
@@ -21,9 +21,7 @@
 INTRCLASS(DEBrush);
 
 #include "style.h"
-
-#define MATCHES(S, A) (gr_stylespec_score(S, A)>0)
-#define MATCHES2(S, A1, A2) (gr_stylespec_score2(S, A1, A2)>0)
+#include "colour.h"
 
 
 typedef void DEBrushExtrasFn(DEBrush *brush, 
