@@ -1,7 +1,7 @@
 /*
  * ion/de/draw.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2005. 
+ * Copyright (c) Tuomo Valkonen 1999-2006. 
  *
  * Ion is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by
@@ -10,13 +10,14 @@
  */
 
 #include <string.h>
+#include <limits.h>
 
 #include <ioncore/global.h>
 #include <ioncore/common.h>
 #include <ioncore/gr.h>
 #include "brush.h"
 #include "font.h"
-#include "misc.h"
+#include "private.h"
 
 #include <X11/extensions/shape.h>
 
@@ -442,16 +443,29 @@ void debrush_set_window_shape(DEBrush *brush, bool rough,
     if(n>MAXSHAPE)
         n=MAXSHAPE;
     
-    for(i=0; i<n; i++){
-        r[i].x=rects[i].x;
-        r[i].y=rects[i].y;
-        r[i].width=rects[i].w;
-        r[i].height=rects[i].h;
+    if(n==0){
+	/* n==0 should clear the shape. As there's absolutely no
+	 * documentation for XShape (as is typical of all sucky X
+	 * extensions), I don't know how the shape should properly 
+	 * be cleared. Thus we just use a huge rectangle.
+	 */
+	n=1;
+	r[0].x=0;
+	r[0].y=0;
+	r[0].width=USHRT_MAX;
+	r[0].height=USHRT_MAX;
+    }else{
+	for(i=0; i<n; i++){
+	    r[i].x=rects[i].x;
+	    r[i].y=rects[i].y;
+	    r[i].width=rects[i].w;
+	    r[i].height=rects[i].h;
+	}
     }
     
     XShapeCombineRectangles(ioncore_g.dpy, brush->win,
                             ShapeBounding, 0, 0, r, n,
-                            ShapeSet, YXBanded);
+                            ShapeSet, Unsorted);
 }
 
 
@@ -536,7 +550,7 @@ static void debrush_clear_clipping_rectangle(DEBrush *brush)
 /*}}}*/
 
 
-/*{{{debrush_begin/end */
+/*{{{ debrush_begin/end */
 
 
 void debrush_begin(DEBrush *brush, const WRectangle *geom, int flags)
