@@ -1,7 +1,7 @@
 /*
  * ion/ioncore/binding.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2004. 
+ * Copyright (c) Tuomo Valkonen 1999-2006. 
  *
  * Ion is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by
@@ -14,10 +14,9 @@
 #include "event.h"
 #include "binding.h"
 #include "global.h"
-#include "objp.h"
+#include <libtu/objp.h>
 #include "regbind.h"
-#include "extl.h"
-#include "defer.h"
+#include <libextl/extl.h>
 
 
 #ifndef CF_NO_LOCK_HACK
@@ -32,6 +31,7 @@
 
 /* */
 
+
 #define N_MODS 8
 
 static const uint modmasks[N_MODS]={
@@ -82,9 +82,8 @@ static void evil_ungrab_button(Display *display, uint button, uint modifiers,
 
 #define CVAL(A, B, V) ( A->V < B->V ? -1 : (A->V > B->V ? 1 : 0))
 
-static int compare_bindings(const void *a_, const void *b_)
+static int compare_bindings(const WBinding *a, const WBinding *b)
 {
-    const WBinding *a=(WBinding*)a_, *b=(WBinding*)b_;
     int r=CVAL(a, b, act);
     if(r==0){
         r=CVAL(a, b, kcb);
@@ -97,20 +96,36 @@ static int compare_bindings(const void *a_, const void *b_)
     }
     return r;
 }
+
+/* This is only used for searching AnyKey etc. */
+static int compare_bindings_ksb(const WBinding *a, const WBinding *b)
+{
+    int r=CVAL(a, b, act);
+    if(r==0){
+        r=CVAL(a, b, ksb);
+        if(r==0){
+            r=CVAL(a, b, state);
+            if(r==0){
+                r=CVAL(a, b, area);
+            }
+        }
+    }
+    return r;
+}
                     
 #undef CVAL
 
 
-static WBindmap *known_bindmaps=NULL;
-
-
 bool init_bindmap(WBindmap *bindmap)
 {
-    bindmap->nbindings=0;
-    bindmap->bindings=NULL;
     bindmap->rbind_list=NULL;
-    bindmap->next_known=NULL;
-    bindmap->prev_known=NULL;
+    bindmap->areamap=NULL;
+    bindmap->nbindings=0;
+    bindmap->bindings=make_rb();
+    if(bindmap->bindings==NULL){
+        warn_err();
+        return FALSE;
+    }
     return TRUE;
 }
 
@@ -124,19 +139,19 @@ WBindmap *create_bindmap()
         return NULL;
     }
     
-    init_bindmap(bindmap);
+    if(!init_bindmap(bindmap)){
+        free(bindmap);
+        return NULL;
+    }
     
     return bindmap;
 }
 
 
-void deinit_binding(WBinding *binding)
+void binding_deinit(WBinding *binding)
 {
-    int i;
-    
     if(binding->submap!=NULL){
-        deinit_bindmap(binding->submap);
-        free(binding->submap);
+        bindmap_destroy(binding->submap);
         binding->submap=NULL;
     }
 
@@ -144,181 +159,190 @@ void deinit_binding(WBinding *binding)
 }
 
 
-void deinit_bindmap(WBindmap *bindmap)
+static void do_destroy_binding(WBinding *binding)
 {
-    int i;
-    WBinding *binding;
+    assert(binding!=NULL);
+    binding_deinit(binding);
+    free(binding);
+}
 
-    UNLINK_ITEM(known_bindmaps, bindmap, next_known, prev_known);
+
+static void bindmap_deinit(WBindmap *bindmap)
+{
+    WBinding *b=NULL;
+    Rb_node node=NULL;
 
     while(bindmap->rbind_list!=NULL){
         region_remove_bindmap(bindmap->rbind_list->reg,
                               bindmap);
     }
         
-    binding=bindmap->bindings;
+    if(bindmap->bindings==NULL)
+        return;
     
-    for(i=0; i<bindmap->nbindings; i++, binding++)
-        deinit_binding(binding);
+    FOR_ALL_BINDINGS(b, node, bindmap->bindings){
+        do_destroy_binding((WBinding*)rb_val(node));
+        bindmap->nbindings--;
+    }
+
+    assert(bindmap->nbindings==0);
     
-    free(bindmap->bindings);
+    rb_free_tree(bindmap->bindings);
     bindmap->bindings=NULL;
-    bindmap->nbindings=0;
 }
 
 
-static void refresh_bindmap(WBindmap *bindmap)
+void bindmap_destroy(WBindmap *bindmap)
+{
+    bindmap_deinit(bindmap);
+    free(bindmap);
+}
+
+
+static void free_map(Rb_node map)
+{
+    Rb_node node;
+    WBinding *b;
+    
+    FOR_ALL_BINDINGS(b, node, map)
+        free(b);
+    
+    rb_free_tree(map);
+}
+
+
+void bindmap_refresh(WBindmap *bindmap)
 {
     WRegBindingInfo *rbind;
-    WBinding *b;
-    int i;
+    Rb_node newtree, node;
+    WBinding *b, *b2;
     
-    for(i=0, b=bindmap->bindings; i<bindmap->nbindings; i++, b++){
-        if(b->act!=ACT_KEYPRESS)
-            continue;
-        for(rbind=bindmap->rbind_list; rbind!=NULL; rbind=rbind->bm_next)
-            rbind_binding_removed(rbind, b, bindmap);
+    if(bindmap->bindings==NULL)
+        return;
+    
+    newtree=make_rb();
+    
+    if(newtree==NULL){
+        warn_err();
+        return;
     }
     
-    for(i=0, b=bindmap->bindings; i<bindmap->nbindings; i++, b++){
-        if(b->act!=ACT_KEYPRESS)
-            continue;
-        b->kcb=XKeysymToKeycode(wglobal.dpy, b->ksb);
+    FOR_ALL_BINDINGS(b, node, bindmap->bindings){
+        b2=ALLOC(WBinding);
+        if(b2==NULL){
+            warn_err();
+            free_map(newtree);
+            return;
+        }
+        
+        *b2=*b;
+
+        if(b->act==BINDING_KEYPRESS){
+            for(rbind=bindmap->rbind_list; rbind!=NULL; rbind=rbind->bm_next)
+                rbind_binding_removed(rbind, b, bindmap);
+            b2->kcb=XKeysymToKeycode(ioncore_g.dpy, b->ksb);
+        }
+        
+        if(!rb_insertg(newtree, b2, b2, (Rb_compfn*)compare_bindings)){
+            warn_err();
+            free(b2);
+            free_map(newtree);
+            return;
+        }
     }
-    
-    qsort((void*)(bindmap->bindings), bindmap->nbindings, sizeof(WBinding), 
-          compare_bindings);
-    
-    for(i=0, b=bindmap->bindings; i<bindmap->nbindings; i++, b++){
-        if(b->act!=ACT_KEYPRESS)
+
+    free_map(bindmap->bindings);
+    bindmap->bindings=newtree;
+
+    FOR_ALL_BINDINGS(b, node, bindmap->bindings){
+        if(b->act!=BINDING_KEYPRESS)
             continue;
         for(rbind=bindmap->rbind_list; rbind!=NULL; rbind=rbind->bm_next)
             rbind_binding_added(rbind, b, bindmap);
         if(b->submap!=NULL)
-            refresh_bindmap(b->submap);
+            bindmap_refresh(b->submap);
     }
 }
 
 
-void refresh_bindings()
+bool bindmap_add_binding(WBindmap *bindmap, const WBinding *b)
 {
-    WBindmap *bindmap;
+    WRegBindingInfo *rbind=NULL;
+    WBinding *binding=NULL;
+    Rb_node node=NULL;
+    int found=0;
     
-    update_modmap();
-    
-    for(bindmap=known_bindmaps; bindmap!=NULL; bindmap=bindmap->next_known)
-        refresh_bindmap(bindmap);
-}
-
-
-bool add_binding(WBindmap *bindmap, const WBinding *b)
-{
-    WBinding *binding;
-    int i, j;
-    
-    if(bindmap==NULL)
-        return FALSE;
-
-    if(bindmap->prev_known==NULL){
-        LINK_ITEM(known_bindmaps, bindmap, next_known, prev_known);
-    }
-
-    binding=bindmap->bindings;
-    
-    for(i=0; i<bindmap->nbindings; i++){
-        switch(compare_bindings((void*)b, (void*)(binding+i))){
-        case 1:
-            continue;
-        case 0:
-            deinit_binding(binding+i);
-            goto subst;
-        }
-        break;
-    }
-
-    binding=REALLOC_N(binding, WBinding, bindmap->nbindings,
-                      bindmap->nbindings+1);
+    /* Handle adding the binding */
+    binding=ALLOC(WBinding);
     
     if(binding==NULL){
         warn_err();
         return FALSE;
     }
     
-    memmove(&(binding[i+1]), &(binding[i]),
-            sizeof(WBinding)*(bindmap->nbindings-i));
+    memcpy(binding, b, sizeof(*b));
     
-    bindmap->bindings=binding;
+    node=rb_find_gkey_n(bindmap->bindings, binding,
+                        (Rb_compfn*)compare_bindings, &found);
+    
+    if(found){
+        if(!rb_insert_a(node, binding, binding)){
+            free(binding);
+            return FALSE;
+        }
+        do_destroy_binding((WBinding*)rb_val(node));
+        rb_delete_node(node);
+        bindmap->nbindings--;
+    }else{
+        if(!rb_insertg(bindmap->bindings, binding, binding, 
+                       (Rb_compfn*)compare_bindings)){
+            free(binding);
+            return FALSE;
+        }
+    }
+
     bindmap->nbindings++;
     
-    {
-        WRegBindingInfo *rbind;
-        for(rbind=bindmap->rbind_list; rbind!=NULL; rbind=rbind->bm_next)
-            rbind_binding_added(rbind, b, bindmap);
-    }
-    
-subst:
-    memcpy(binding+i, b, sizeof(WBinding));
-    
+    for(rbind=bindmap->rbind_list; rbind!=NULL; rbind=rbind->bm_next)
+        rbind_binding_added(rbind, binding, bindmap);
+
     return TRUE;
 }
 
 
-bool remove_binding(WBindmap *bindmap, const WBinding *b)
+bool bindmap_remove_binding(WBindmap *bindmap, const WBinding *b)
 {
-    WBinding *binding;
-    int i, j;
+    WRegBindingInfo *rbind=NULL;
+    WBinding *binding=NULL;
+    Rb_node node=NULL;
+    int found=0;
     
-    if(bindmap==NULL)
+    if(bindmap->bindings==NULL)
         return FALSE;
-
-    binding=bindmap->bindings;
     
-    for(i=0; i<bindmap->nbindings; i++){
-        switch(compare_bindings((void*)b, (void*)(binding+i))){
-        case 1:
-            continue;
-        case 0:
-            goto rmove;
-        }
-        break;
-    }
+    node=rb_find_gkey_n(bindmap->bindings, b, (Rb_compfn*)compare_bindings,
+                        &found);
     
-    return FALSE;
-
-rmove:
-    deinit_binding(binding+i);
-
-    {
-        WRegBindingInfo *rbind;
-        for(rbind=bindmap->rbind_list; rbind!=NULL; rbind=rbind->bm_next){
-            rbind_binding_removed(rbind, b, bindmap);
-        }
-    }
-
-    memmove(&(binding[i]), &(binding[i+1]),
-            sizeof(WBinding)*(bindmap->nbindings-i));
+    if(!found)
+        return FALSE;
+    
+    binding=(WBinding*)rb_val(node);
+    
+    for(rbind=bindmap->rbind_list; rbind!=NULL; rbind=rbind->bm_next)
+        rbind_binding_removed(rbind, binding, bindmap);
 
+    do_destroy_binding(binding);
+    rb_delete_node(node);
+    
     bindmap->nbindings--;
-    
-    if(bindmap->nbindings==0){
-        free(binding);
-        bindmap->bindings=NULL;
-    }else{
-        binding=REALLOC_N(binding, WBinding, bindmap->nbindings,
-                          bindmap->nbindings);
-        if(binding==NULL)
-            warn_err();
-        else
-            bindmap->bindings=binding;
-    }
-    
+
     return TRUE;
 }
 
 
-void init_bindings()
+void ioncore_init_bindings()
 {
-    modmap=XGetModifierMapping(wglobal.dpy);
+    modmap=XGetModifierMapping(ioncore_g.dpy);
     
     assert(modmap!=NULL);
 
@@ -328,9 +352,9 @@ void init_bindings()
 }
 
 
-void update_modmap()
+void ioncore_update_modmap()
 {
-    XModifierKeymap *nm=XGetModifierMapping(wglobal.dpy);
+    XModifierKeymap *nm=XGetModifierMapping(ioncore_g.dpy);
     
     if(nm!=NULL){
         XFreeModifiermap(modmap);
@@ -342,92 +366,108 @@ void update_modmap()
 /* */
 
 
-void grab_binding(const WBinding *binding, Window win)
+void binding_grab_on(const WBinding *binding, Window win)
 {
-    if(binding->act==ACT_KEYPRESS){
-#ifndef CF_HACK_IGNORE_EVIL_LOCKS            
-        XGrabKey(wglobal.dpy, binding->kcb, binding->state, win,
+    if(binding->act==BINDING_KEYPRESS && binding->kcb!=0){
+#ifndef CF_HACK_IGNORE_EVIL_LOCKS
+        XGrabKey(ioncore_g.dpy, binding->kcb, binding->state, win,
                  True, GrabModeAsync, GrabModeAsync);
-#else        
-        evil_grab_key(wglobal.dpy, binding->kcb, binding->state, win,
+#else
+        evil_grab_key(ioncore_g.dpy, binding->kcb, binding->state, win,
                       True, GrabModeAsync, GrabModeAsync);
-#endif            
+#endif
     }
     
-    if(binding->act!=ACT_BUTTONPRESS &&
-       binding->act!=ACT_BUTTONCLICK &&
-       binding->act!=ACT_BUTTONDBLCLICK &&
-       binding->act!=ACT_BUTTONMOTION)
+    if(binding->act!=BINDING_BUTTONPRESS &&
+       binding->act!=BINDING_BUTTONCLICK &&
+       binding->act!=BINDING_BUTTONDBLCLICK &&
+       binding->act!=BINDING_BUTTONMOTION)
         return;
     
     if(binding->state==0)
         return;
     
-#ifndef CF_HACK_IGNORE_EVIL_LOCKS            
-    XGrabButton(wglobal.dpy, binding->kcb, binding->state, win,
-                True, GRAB_POINTER_MASK, GrabModeAsync, GrabModeAsync,
+#ifndef CF_HACK_IGNORE_EVIL_LOCKS
+    XGrabButton(ioncore_g.dpy, binding->kcb, binding->state, win,
+                True, IONCORE_EVENTMASK_PTRGRAB, GrabModeAsync, GrabModeAsync,
                 None, None);
-#else            
-    evil_grab_button(wglobal.dpy, binding->kcb, binding->state, win,
-                     True, GRAB_POINTER_MASK, GrabModeAsync, GrabModeAsync,
+#else
+    evil_grab_button(ioncore_g.dpy, binding->kcb, binding->state, win,
+                     True, IONCORE_EVENTMASK_PTRGRAB, GrabModeAsync, GrabModeAsync,
                      None, None);
 #endif
 }
 
 
-void ungrab_binding(const WBinding *binding, Window win)
+void binding_ungrab_on(const WBinding *binding, Window win)
 {
-    if(binding->act==ACT_KEYPRESS){
+    if(binding->act==BINDING_KEYPRESS){
 #ifndef CF_HACK_IGNORE_EVIL_LOCKS
-        XUngrabKey(wglobal.dpy, binding->kcb, binding->state, win);
+        XUngrabKey(ioncore_g.dpy, binding->kcb, binding->state, win);
 #else
-        evil_ungrab_key(wglobal.dpy, binding->kcb, binding->state, win);
+        evil_ungrab_key(ioncore_g.dpy, binding->kcb, binding->state, win);
 #endif
     }
     
-    if(binding->act!=ACT_BUTTONPRESS &&
-       binding->act!=ACT_BUTTONCLICK &&
-       binding->act!=ACT_BUTTONDBLCLICK &&
-       binding->act!=ACT_BUTTONMOTION)
+    if(binding->act!=BINDING_BUTTONPRESS &&
+       binding->act!=BINDING_BUTTONCLICK &&
+       binding->act!=BINDING_BUTTONDBLCLICK &&
+       binding->act!=BINDING_BUTTONMOTION)
         return;
     
     if(binding->state==0)
         return;
 
 #ifndef CF_HACK_IGNORE_EVIL_LOCKS
-    XUngrabButton(wglobal.dpy, binding->kcb, binding->state, win);
+    XUngrabButton(ioncore_g.dpy, binding->kcb, binding->state, win);
 #else
-    evil_ungrab_button(wglobal.dpy, binding->kcb, binding->state, win);
+    evil_ungrab_button(ioncore_g.dpy, binding->kcb, binding->state, win);
 #endif
 }
 
 
-#if 0
-void grab_bindings(WBindmap *bindmap, Window win)
-{
-    WBinding *binding;
-    int i;
-    
-    binding=bindmap->bindings;
-    for(i=0; i<bindmap->nbindings; i++, binding++)
-        grab_binding(binding, win);
-}
-#endif
-
-
 /* */
 
 
 static WBinding *search_binding(WBindmap *bindmap, WBinding *binding)
 {
-    return (WBinding*)bsearch((void*)binding, (void*)(bindmap->bindings),
-                              bindmap->nbindings, sizeof(WBinding),
-                              compare_bindings);
+    Rb_node node;
+    int found=0;
+
+    if(bindmap->bindings==NULL)
+        return NULL;
+    
+    node=rb_find_gkey_n(bindmap->bindings, binding,
+                        (Rb_compfn*)compare_bindings, &found);
+    
+    if(found==0)
+        return NULL;
+    
+    return (WBinding*)rb_val(node);
 }
 
 
-static WBinding *do_lookup_binding(WBindmap *bindmap,
-                                   int act, uint state, uint kcb, int area)
+static WBinding *search_binding_ksb(WBindmap *bindmap, WBinding *binding)
+{
+    Rb_node node;
+    int found=0;
+
+    if(bindmap->bindings==NULL)
+        return NULL;
+    
+    node=rb_find_gkey_n(bindmap->bindings, binding,
+                        (Rb_compfn*)compare_bindings_ksb, &found);
+    
+    if(found==0)
+        return NULL;
+    
+    return (WBinding*)rb_val(node);
+}
+
+
+static WBinding *do_bindmap_lookup_binding(WBindmap *bindmap,
+                                           int act, uint state, 
+                                           uint kcb, int area)
 {
     WBinding *binding, tmp;
 
@@ -452,12 +492,13 @@ static WBinding *do_lookup_binding(WBindmap *bindmap,
 
         if(binding==NULL){
             tmp.state=state;
-            tmp.kcb=(act==ACT_KEYPRESS ? AnyKey : AnyButton);
-            binding=search_binding(bindmap, &tmp);
+            tmp.ksb=(act==BINDING_KEYPRESS ? AnyKey : AnyButton);
+            
+            binding=search_binding_ksb(bindmap, &tmp);
 
             if(binding==NULL){
                 tmp.state=AnyModifier;
-                binding=search_binding(bindmap, &tmp);
+                binding=search_binding_ksb(bindmap, &tmp);
             }
         }
     }
@@ -466,21 +507,22 @@ static WBinding *do_lookup_binding(WBindmap *bindmap,
 }
 
 
-WBinding *lookup_binding(WBindmap *bindmap, int act, uint state, uint kcb)
+WBinding *bindmap_lookup_binding(WBindmap *bindmap,
+                                 int act, uint state, uint kcb)
 {
-    return do_lookup_binding(bindmap, act, state, kcb, 0);
+    return do_bindmap_lookup_binding(bindmap, act, state, kcb, 0);
 }
 
 
-WBinding *lookup_binding_area(WBindmap *bindmap,
-                              int act, uint state, uint kcb, int area)
+WBinding *bindmap_lookup_binding_area(WBindmap *bindmap,
+                                      int act, uint state, uint kcb, int area)
 {
     WBinding *binding;
     
-    binding=do_lookup_binding(bindmap, act, state, kcb, area);
+    binding=do_bindmap_lookup_binding(bindmap, act, state, kcb, area);
     
     if(binding==NULL)
-        binding=do_lookup_binding(bindmap, act, state, kcb, 0);
+        binding=do_bindmap_lookup_binding(bindmap, act, state, kcb, 0);
     
     return binding;
 }
@@ -491,7 +533,7 @@ WBinding *lookup_binding_area(WBindmap *bindmap,
  */
 
 
-int unmod(int state, int keycode)
+int ioncore_unmod(int state, int keycode)
 {
     int j;
     
@@ -508,13 +550,13 @@ int unmod(int state, int keycode)
 }
 
 
-int modstate()
+int ioncore_modstate()
 {
 	char keys[32];
 	int state=0;
 	int j;
 	
-	XQueryKeymap(wglobal.dpy, keys);
+	XQueryKeymap(ioncore_g.dpy, keys);
     
 	for(j=0; j<N_MODS*modmap->max_keypermod; j++){
         int a=(modmap->modifiermap[j]&7);
@@ -532,7 +574,7 @@ int modstate()
 }
 
 
-bool ismod(int keycode)
+bool ioncore_ismod(int keycode)
 {
     int j;
     
@@ -553,7 +595,7 @@ static void lookup_evil_locks()
     int i, j;
     
     for(i=0; i<N_LOOKUPEVIL; i++)
-        keycodes[i]=XKeysymToKeycode(wglobal.dpy, evillocks[i]);
+        keycodes[i]=XKeysymToKeycode(ioncore_g.dpy, evillocks[i]);
     
     for(j=0; j<N_MODS*modmap->max_keypermod; j++){
         for(i=0; i<N_LOOKUPEVIL; i++){
