@@ -42,7 +42,7 @@ DECLSTRUCT(WBinding){
     uint state;
     uint act;
     int area;
-    bool waitrel;
+    bool wait;
     WBindmap *submap;
     ExtlFn func;
 };
@@ -55,6 +55,7 @@ DECLSTRUCT(WRegBindingInfo){
     WRegBindingInfo *bm_next, *bm_prev;
     WRegion *reg;
     WRegion *owner;
+    int tmp;
 };
 
 
