@@ -1,7 +1,7 @@
 /*
  * ion/ioncore/bindmaps.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2005. 
+ * Copyright (c) Tuomo Valkonen 1999-2006. 
  *
  * Ion is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by
@@ -25,8 +25,17 @@
 
 WBindmap *ioncore_rootwin_bindmap=NULL;
 WBindmap *ioncore_mplex_bindmap=NULL;
+WBindmap *ioncore_mplex_toplevel_bindmap=NULL;
 WBindmap *ioncore_frame_bindmap=NULL;
+WBindmap *ioncore_frame_toplevel_bindmap=NULL;
+WBindmap *ioncore_frame_floating_bindmap=NULL;
+WBindmap *ioncore_frame_tiled_bindmap=NULL;
+WBindmap *ioncore_frame_transient_bindmap=NULL;
 WBindmap *ioncore_moveres_bindmap=NULL;
+WBindmap *ioncore_group_bindmap=NULL;
+WBindmap *ioncore_groupcw_bindmap=NULL;
+WBindmap *ioncore_groupws_bindmap=NULL;
+WBindmap *ioncore_clientwin_bindmap=NULL;
 
 static Rb_node known_bindmaps=NULL;
 
@@ -49,8 +58,17 @@ void ioncore_deinit_bindmaps()
 {
     DO_FREE(rootwin, "WScreen");
     DO_FREE(mplex, "WMPlex");
+    DO_FREE(mplex_toplevel, "WMPlex.toplevel");
     DO_FREE(frame, "WFrame");
+    DO_FREE(frame_toplevel, "WFrame.toplevel");
+    DO_FREE(frame_floating, "WFrame.floating");
+    DO_FREE(frame_tiled, "WFrame.tiled");
+    DO_FREE(frame_transient, "WFrame.transient");
     DO_FREE(moveres, "WMoveresMode");
+    DO_FREE(group, "WGroup");
+    DO_FREE(groupcw, "WGroupCW");
+    DO_FREE(groupws, "WGroupWS");
+    DO_FREE(clientwin, "WClientWin");
     rb_free_tree(known_bindmaps);
     known_bindmaps=NULL;
 }
@@ -70,8 +88,17 @@ bool ioncore_init_bindmaps()
     
     DO_ALLOC(rootwin, "WScreen", NULL);
     DO_ALLOC(mplex, "WMPlex", NULL);
+    DO_ALLOC(mplex_toplevel, "WMPlex.toplevel", NULL);
     DO_ALLOC(frame, "WFrame", frame_areas);
+    DO_ALLOC(frame_toplevel, "WFrame.toplevel", frame_areas);
+    DO_ALLOC(frame_floating, "WFrame.floating", frame_areas);
+    DO_ALLOC(frame_tiled, "WFrame.tiled", frame_areas);
+    DO_ALLOC(frame_transient, "WFrame.transient", frame_areas);
     DO_ALLOC(moveres, "WMoveresMode", NULL);
+    DO_ALLOC(group, "WGroup", NULL);
+    DO_ALLOC(groupcw, "WGroupCW", NULL);
+    DO_ALLOC(groupws, "WGroupWS", NULL);
+    DO_ALLOC(clientwin, "WClientWin", NULL);
     
     return TRUE;
 }
