@@ -20,7 +20,6 @@
 
 INTRCLASS(WClientWin);
 INTRCLASS(WFrame);
-INTRCLASS(WGenWS);
 INTRCLASS(WInfoWin);
 INTRCLASS(WMPlex);
 INTRCLASS(WRegion);
@@ -28,8 +27,19 @@ INTRCLASS(WMoveresMode);
 INTRCLASS(WRootWin);
 INTRCLASS(WScreen);
 INTRCLASS(WWindow);
+INTRCLASS(WGroup);
+INTRCLASS(WGroupCW);
+INTRCLASS(WGroupWS);
 
 INTRCLASS(WPHolder);
 INTRCLASS(WMPlexPHolder);
 
+INTRSTRUCT(WStacking);
+INTRSTRUCT(WLListNode);
+INTRSTRUCT(WStackingIterTmp);
+
+INTRSTRUCT(WSubmapState);
+
+INTRSTRUCT(WRegionAttachData);
+
 #endif /* ION_IONCORE_CLASSES_H */
