@@ -47,16 +47,10 @@ DECLCLASS(WClientWin){
     int event_mask;
     Window win;
     
-    WFitParams last_fp;
-    
     int orig_bw;
 
     Window transient_for;
     
-    WLListNode *transient_list;
-    
-    WClientWin *g_cwin_next, *g_cwin_prev;
-    
     Colormap cmap;
     Colormap *cmaps;
     Window *cmapwins;
@@ -67,9 +61,6 @@ DECLCLASS(WClientWin){
     WPHolder *fs_pholder;
     
     ExtlTab proptab;
-
-    WSizePolicy szplcy;           /* size policy (when there is slack) */
-    WSizePolicy transient_szplcy; /* default transient size policy */
 };
 
 
@@ -82,17 +73,6 @@ extern bool clientwin_rqclose(WClientWin *cwin, bool relocate_ignored);
 
 extern void clientwin_tfor_changed(WClientWin *cwin);
 
-extern bool clientwin_attach_transient(WClientWin *cwin, WRegion *transient);
-extern WPHolder *clientwin_prepare_manage(WClientWin *cwin, 
-                                          const WClientWin *cwin2,
-                                          const WManageParams *param, 
-                                          int redir);
-
-extern WPHolder *clientwin_prepare_manage_transient(WClientWin *cwin, 
-                                                    const WClientWin *transient,
-                                                    const WManageParams *param,
-                                                    int unused);
-
 extern void clientwin_get_set_name(WClientWin *cwin);
 
 extern void clientwin_handle_configure_request(WClientWin *cwin,
