@@ -14,18 +14,17 @@
 
 #include <libtu/map.h>
 #include <libtu/minmax.h>
+#include <libtu/objp.h>
+#include <libextl/readconfig.h>
 
 #include "common.h"
 #include "global.h"
-#include <libextl/readconfig.h>
 #include "modules.h"
 #include "rootwin.h"
 #include "bindmaps.h"
 #include "kbresize.h"
-#include <libextl/readconfig.h>
-
-
-char *ioncore_default_ws_type=NULL;
+#include "reginfo.h"
+#include "group-ws.h"
 
 
 /*EXTL_DOC
@@ -47,8 +46,6 @@ char *ioncore_default_ws_type=NULL;
  *  \var{frame_add_last} & (boolean) Add new regions in frames last instead
  *                        of after current region. \\
  *  \var{dblclick_delay} & (integer) Delay between clicks of a double click.\\
- *  \var{default_ws_type} & (string) Default workspace type for operations
- *                         that create a workspace.\\
  *  \var{kbresize_delay} & (integer) Delay in milliseconds for ending keyboard
  *                         resize mode after inactivity. \\
  *  \var{kbresize_t_max} & (integer) Controls keyboard resize acceleration. 
@@ -57,6 +54,12 @@ char *ioncore_default_ws_type=NULL;
  *  \var{kbresize_step} & (floating point) See below. \\
  *  \var{kbresize_maxacc} & (floating point) See below. \\
  *  \var{framed_transients} & (boolean) Put transients in nested frames. \\
+ *  \var{float_placement_method} & (string) How to place floating frames.
+ *                          One of ''udlr'' (up-down, then left-right), 
+ *                          ''lrud'' (left-right, then up-down) or ''random''. \\
+ *  \var{default_ws_layout} & (table) Default workspace layout;
+ *                          The \var{managed} field of \type{WGroup}
+ *                          load parameters. \\
  * \end{tabularx}
  * 
  * When a keyboard resize function is called, and at most \var{kbresize_t_max} 
@@ -72,6 +75,7 @@ void ioncore_set(ExtlTab tab)
 {
     int dd, rd;
     char *wst;
+    ExtlTab t;
     
     extl_table_gets_b(tab, "opaque_resize", &(ioncore_g.opaque_resize));
     extl_table_gets_b(tab, "warp", &(ioncore_g.warp_enabled));
@@ -82,12 +86,10 @@ void ioncore_set(ExtlTab tab)
     
     if(extl_table_gets_i(tab, "dblclick_delay", &dd))
         ioncore_g.dblclick_delay=maxof(0, dd);
-    if(extl_table_gets_s(tab, "default_ws_type", &wst)){
-        if(ioncore_default_ws_type!=NULL)
-            free(ioncore_default_ws_type);
-        ioncore_default_ws_type=wst;
-    }
+    
     ioncore_set_moveres_accel(tab);
+    
+    ioncore_groupws_set(tab);
 }
 
 
@@ -105,10 +107,12 @@ ExtlTab ioncore_get()
     extl_table_sets_b(tab, "switchto", ioncore_g.switchto_new);
     extl_table_sets_i(tab, "dblclick_delay", ioncore_g.dblclick_delay);
     extl_table_sets_b(tab, "screen_notify", ioncore_g.screen_notify);
-    extl_table_sets_s(tab, "default_ws_type", ioncore_default_ws_type);
     extl_table_sets_b(tab, "frame_add_last", ioncore_g.frame_add_last);
+    
     ioncore_get_moveres_accel(tab);
     
+    ioncore_groupws_get(tab);
+    
     return tab;
 }
         
