@@ -1,7 +1,7 @@
 /*
  * ion/ioncore/conf.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2004. 
+ * Copyright (c) Tuomo Valkonen 1999-2006. 
  *
  * Ion is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by
@@ -10,8 +10,8 @@
  */
 
 #ifndef ION_IONCORE_CONF_H
-#define WMCORe_CONF_H
+#define ION_IONCORE_CONF_H
 
-extern bool ioncore_read_config(const char *cfgfile);
+extern bool ioncore_read_main_config(const char *cfgfile);
 
 #endif /* ION_IONCORE_CONF_H */
