@@ -63,7 +63,7 @@ bool ioncore_handle_event(XEvent *ev)
         ioncore_handle_property(&(ev->xproperty));
         break;
     CASE_EVENT(FocusIn)
-        ioncore_handle_focus_in(&(ev->xfocus));
+        ioncore_handle_focus_in(&(ev->xfocus), FALSE);
         break;
     CASE_EVENT(FocusOut)
         ioncore_handle_focus_out(&(ev->xfocus));
@@ -308,53 +308,43 @@ void ioncore_handle_expose(const XExposeEvent *ev)
 void ioncore_handle_enter_window(XEvent *ev)
 {
     XEnterWindowEvent *eev=&(ev->xcrossing);
-    WRegion *reg=NULL, *freg=NULL, *mgr=NULL;
-    bool more=TRUE;
+    WRegion *reg=NULL;
     
     if(ioncore_g.input_mode!=IONCORE_INPUTMODE_NORMAL)
         return;
-
-    do{
-        if(eev->mode!=NotifyNormal && !ioncore_g.warp_enabled)
-            continue;
-        /*if(eev->detail==NotifyNonlinearVirtual)
-            continue;*/
-
-        reg=XWINDOW_REGION_OF_T(eev->window, WRegion);
-        
-        if(reg==NULL)
-            continue;
-
-        D(fprintf(stderr, "E: %p %s %d %d\n", reg, OBJ_TYPESTR(reg),
-                  eev->mode, eev->detail));
-        
-        /* If an EnterWindow event was already found that we're going to
-         * handle, do not note subsequent events if they are into an 
-         * ancestor of the window of freg.
-         */
-        if(freg!=NULL){
-            WRegion *r2=freg;
-            while(r2!=NULL){
-                if(r2==reg)
-                    break;
-                r2=REGION_PARENT_REG(r2);
-            }
-            if(r2!=NULL)
-                continue;
-        }
-        
-        if(!REGION_IS_ACTIVE(reg))
-            freg=reg;
-    }while(freg!=NULL && XCheckMaskEvent(ioncore_g.dpy, EnterWindowMask, ev));
-
-    if(freg==NULL)
+    
+    if(ioncore_await_focus())
         return;
     
-    if(!region_skip_focus(freg)){
-        region_goto_flags(freg, (REGION_GOTO_FOCUS|
-                                 REGION_GOTO_NOWARP|
-                                 REGION_GOTO_ENTERWINDOW));
+    if(eev->mode!=NotifyNormal && !ioncore_g.warp_enabled)
+        return;
+
+    reg=XWINDOW_REGION_OF_T(eev->window, WRegion);
+    
+    if(reg==NULL)
+        return;
+
+    if(REGION_IS_ACTIVE(reg))
+        return;
+    
+    if(region_skip_focus(reg))
+        return;
+    
+    /* If a child of 'reg' is to be focused, do not process this
+     * event.
+     */
+    if(ioncore_g.focus_next!=NULL){
+        WRegion *r2=ioncore_g.focus_next;
+        while(r2!=NULL){
+            if(r2==reg)
+                return;
+            r2=REGION_PARENT_REG(r2);
+        }
     }
+        
+    region_goto_flags(reg, (REGION_GOTO_FOCUS|
+                            REGION_GOTO_NOWARP|
+                            REGION_GOTO_ENTERWINDOW));
 }
 
 
@@ -372,7 +362,7 @@ static bool pointer_in_root(Window root1)
 
 
 
-void ioncore_handle_focus_in(const XFocusChangeEvent *ev)
+void ioncore_handle_focus_in(const XFocusChangeEvent *ev, bool skip)
 {
     WRegion *reg;
     WWindow *wwin, *tmp;
@@ -398,7 +388,8 @@ void ioncore_handle_focus_in(const XFocusChangeEvent *ev)
         if((ev->detail==NotifyPointerRoot || ev->detail==NotifyDetailNone) &&
            pointer_in_root(ev->window) && ioncore_g.focus_next==NULL){
             /* Restore focus */
-            region_set_focus(reg);
+            if(!skip)
+                region_set_focus(reg);
             return;
         }
         /*return;*/
@@ -502,7 +493,7 @@ void ioncore_handle_buttonpress(XEvent *ev)
             ioncore_handle_grabs(ev);
             break;
         CASE_EVENT(FocusIn)
-            ioncore_handle_focus_in(&(ev->xfocus));
+            ioncore_handle_focus_in(&(ev->xfocus), FALSE);
             break;
         CASE_EVENT(FocusOut)
             ioncore_handle_focus_out(&(ev->xfocus));
