@@ -14,7 +14,7 @@
 #include <libtu/obj.h>
 #include <libtu/objp.h>
 #include <libtu/minmax.h>
-#include <libtu/stringstore.h>
+#include <libtu/map.h>
 #include <libmainloop/defer.h>
 
 #include "common.h"
@@ -37,7 +37,6 @@
 #include "bindmaps.h"
 #include "regbind.h"
 #include "gr.h"
-#include "genws.h"
 #include "activity.h"
 #include "llist.h"
 
@@ -48,21 +47,26 @@ extern void frame_initialise_gr(WFrame *frame);
 static bool frame_initialise_titles(WFrame *frame);
 static void frame_free_titles(WFrame *frame);
 
+static void frame_add_mode_bindmaps(WFrame *frame);
+
 
 WHook *frame_managed_changed_hook=NULL;
 
+#define IS_FLOATING_MODE(FRAME) \
+    ((FRAME)->mode==FRAME_MODE_FLOATING || (FRAME)->mode==FRAME_MODE_TRANSIENT)
+#define FORWARD_CWIN_RQGEOM(FRAME) IS_FLOATING_MODE(FRAME)
+#define USE_MINMAX(FRAME) IS_FLOATING_MODE(FRAME)
+#define DEST_EMPTY(FRAME) IS_FLOATING_MODE(FRAME)
+
 
 /*{{{ Destroy/create frame */
 
 
 bool frame_init(WFrame *frame, WWindow *parent, const WFitParams *fp,
-                const char *style)
+                WFrameMode mode)
 {
     WRectangle mg;
 
-    if(style==NULL)
-        style="frame";
-    
     frame->flags=0;
     frame->saved_w=0;
     frame->saved_h=0;
@@ -72,24 +76,25 @@ bool frame_init(WFrame *frame, WWindow *parent, const WFitParams *fp,
     frame->titles=NULL;
     frame->titles_n=0;
     frame->bar_h=0;
+    frame->bar_w=fp->g.w;
     frame->tr_mode=GR_TRANSPARENCY_DEFAULT;
     frame->brush=NULL;
     frame->bar_brush=NULL;
-    frame->style=stringstore_alloc(style);
+    frame->mode=mode;
+    frame->tab_min_w=0;
+    frame->bar_max_width_q=1.0;
     
-    if(frame->style==STRINGID_NONE)
+    if(!mplex_init((WMPlex*)frame, parent, fp))
         return FALSE;
-
-    if(!mplex_init((WMPlex*)frame, parent, fp)){
-        stringstore_free(frame->style);
-        return FALSE;
-    }
     
     frame_initialise_gr(frame);
     frame_initialise_titles(frame);
     
     region_add_bindmap((WRegion*)frame, ioncore_frame_bindmap);
-
+    region_add_bindmap((WRegion*)frame, ioncore_frame_toplevel_bindmap);
+    
+    frame_add_mode_bindmaps(frame);
+    
     mplex_managed_geom((WMPlex*)frame, &mg);
     
     if(mg.h<=1)
@@ -101,9 +106,9 @@ bool frame_init(WFrame *frame, WWindow *parent, const WFitParams *fp,
 }
 
 
-WFrame *create_frame(WWindow *parent, const WFitParams *fp, const char *style)
+WFrame *create_frame(WWindow *parent, const WFitParams *fp, WFrameMode mode)
 {
-    CREATEOBJ_IMPL(WFrame, frame, (p, parent, fp, style));
+    CREATEOBJ_IMPL(WFrame, frame, (p, parent, fp, mode));
 }
 
 
@@ -111,7 +116,6 @@ void frame_deinit(WFrame *frame)
 {
     frame_free_titles(frame);
     frame_release_brushes(frame);
-    stringstore_free(frame->style);
     mplex_deinit((WMPlex*)frame);
 }
 
@@ -119,10 +123,102 @@ void frame_deinit(WFrame *frame)
 /*}}}*/
 
 
+/*{{{ Mode switching */
+
+
+static void frame_add_mode_bindmaps(WFrame *frame)
+{
+    WFrameMode mode=frame->mode;
+    
+    if(mode==FRAME_MODE_TILED || mode==FRAME_MODE_TILED_ALT){
+	region_add_bindmap((WRegion*)frame, ioncore_frame_toplevel_bindmap);
+        region_add_bindmap((WRegion*)frame, ioncore_frame_tiled_bindmap);
+    }else if(mode==FRAME_MODE_FLOATING){
+	region_add_bindmap((WRegion*)frame, ioncore_frame_toplevel_bindmap);
+        region_add_bindmap((WRegion*)frame, ioncore_frame_floating_bindmap);
+    }else if(mode==FRAME_MODE_TRANSIENT){
+        region_add_bindmap((WRegion*)frame, ioncore_frame_transient_bindmap);
+    }
+}
+
+
+void frame_set_mode(WFrame *frame, WFrameMode mode)
+{
+    if(frame->mode==mode)
+        return;
+    
+    frame_clear_shape(frame);
+    
+    frame_release_brushes(frame);
+    
+    region_remove_bindmap((WRegion*)frame, ioncore_frame_toplevel_bindmap);
+    region_remove_bindmap((WRegion*)frame, ioncore_frame_tiled_bindmap);
+    region_remove_bindmap((WRegion*)frame, ioncore_frame_floating_bindmap);
+    region_remove_bindmap((WRegion*)frame, ioncore_frame_transient_bindmap);
+    
+    frame->mode=mode;
+
+    frame_add_mode_bindmaps(frame);
+    
+    frame_initialise_gr(frame);
+    
+    mplex_fit_managed(&frame->mplex);
+    frame_recalc_bar(frame);
+    frame_set_background(frame, TRUE);
+}
+
+
+WFrameMode frame_mode(WFrame *frame)
+{
+    return frame->mode;
+}
+
+
+StringIntMap frame_modes[]={
+    {"tiled", FRAME_MODE_TILED},
+    {"tiled-alt", FRAME_MODE_TILED_ALT},
+    {"floating", FRAME_MODE_FLOATING},
+    {"transient", FRAME_MODE_TRANSIENT},
+    END_STRINGINTMAP
+};
+
+
+/*EXTL_DOC
+ * Get frame mode.
+ */
+EXTL_EXPORT_AS(WFrame, mode)
+const char *frame_mode_extl(WFrame *frame)
+{
+    return stringintmap_key(frame_modes, frame->mode, NULL);
+}
+
+
+/*EXTL_DOC
+ * Set frame mode.
+ */
+EXTL_EXPORT_AS(WFrame, set_mode)
+bool frame_set_mode_extl(WFrame *frame, const char *modestr)
+{
+    WFrameMode mode;
+    int idx;
+    
+    idx=stringintmap_ndx(frame_modes, modestr);
+    if(idx<0)
+	return FALSE;
+    
+    frame_set_mode(frame, frame_modes[idx].value);
+    
+    return TRUE;
+}
+
+
+/*}}}*/
+
+
 /*{{{ Tabs */
 
 
-int frame_tab_at_x(const WFrame *frame, int x)
+int frame_tab_at_x(WFrame *frame, int x)
 {
     WRectangle bg;
     int tab, tx;
@@ -144,7 +240,7 @@ int frame_tab_at_x(const WFrame *frame, int x)
 }
 
 
-int frame_nth_tab_x(const WFrame *frame, int n)
+int frame_nth_tab_x(WFrame *frame, int n)
 {
     uint x=0;
     int i;
@@ -156,7 +252,7 @@ int frame_nth_tab_x(const WFrame *frame, int n)
 }
 
 
-static int frame_nth_tab_w_iw(const WFrame *frame, int n, bool inner)
+static int frame_nth_tab_w_iw(WFrame *frame, int n, bool inner)
 {
     WRectangle bg;
     GrBorderWidths bdw=GR_BORDER_WIDTHS_INIT;
@@ -190,13 +286,13 @@ static int frame_nth_tab_w_iw(const WFrame *frame, int n, bool inner)
 }
 
 
-int frame_nth_tab_w(const WFrame *frame, int n)
+int frame_nth_tab_w(WFrame *frame, int n)
 {
     return frame_nth_tab_w_iw(frame, n, FALSE);
 }
 
 
-int frame_nth_tab_iw(const WFrame *frame, int n)
+int frame_nth_tab_iw(WFrame *frame, int n)
 {
     return frame_nth_tab_w_iw(frame, n, TRUE);
 }
@@ -250,7 +346,7 @@ void frame_update_attr_nth(WFrame *frame, int i)
     if(i<0 || i>=frame->titles_n)
         return;
 
-    update_attr(frame, i, mplex_lnth((WMPlex*)frame, 1, i));
+    update_attr(frame, i, mplex_mx_nth((WMPlex*)frame, i));
 }
 
 
@@ -260,7 +356,7 @@ static void update_attrs(WFrame *frame)
     WRegion *sub;
     WLListIterTmp tmp;
     
-    FRAME_L1_FOR_ALL(sub, frame, tmp){
+    FRAME_MX_FOR_ALL(sub, frame, tmp){
         update_attr(frame, i, sub);
         i++;
     }
@@ -290,7 +386,7 @@ static void do_init_title(WFrame *frame, int i, WRegion *sub)
     update_attr(frame, i, sub);
 }
 
-    
+
 static bool frame_initialise_titles(WFrame *frame)
 {
     int i, n=FRAME_MCOUNT(frame);
@@ -311,7 +407,7 @@ static bool frame_initialise_titles(WFrame *frame)
         WLListIterTmp tmp;
         WRegion *sub;
         i=0;
-        FRAME_L1_FOR_ALL(sub, frame, tmp){
+        FRAME_MX_FOR_ALL(sub, frame, tmp){
             do_init_title(frame, i, sub);
             i++;
         }
@@ -372,7 +468,7 @@ bool frame_fitrep(WFrame *frame, WWindow *par, const WFitParams *fp)
 }
 
 
-void frame_resize_hints(WFrame *frame, XSizeHints *hints_ret)
+void frame_size_hints(WFrame *frame, WSizeHints *hints_ret)
 {
     WRectangle subgeom;
     WLListIterTmp tmp;
@@ -386,26 +482,29 @@ void frame_resize_hints(WFrame *frame, XSizeHints *hints_ret)
 
     if(FRAME_CURRENT(frame)!=NULL){
         region_size_hints(FRAME_CURRENT(frame), hints_ret);
-        if(!(frame->flags&FRAME_SZH_USEMINMAX))
-            hints_ret->flags&=PResizeInc|PBaseSize;
+        if(!USE_MINMAX(frame)){
+            hints_ret->max_set=0;
+            hints_ret->min_set=0;
+            /*hints_ret->no_constrain=TRUE;*/
+        }
     }else{
-        hints_ret->flags=0;
+        sizehints_clear(hints_ret);
     }
     
-    FRAME_L1_FOR_ALL(sub, frame, tmp){
-        xsizehints_adjust_for(hints_ret, sub);
+    FRAME_MX_FOR_ALL(sub, frame, tmp){
+        sizehints_adjust_for(hints_ret, sub);
     }
     
-    if(!(hints_ret->flags&PBaseSize)){
+    if(!hints_ret->base_set){
         hints_ret->base_width=0;
         hints_ret->base_height=0;
-        hints_ret->flags|=PBaseSize;
+        hints_ret->base_set=TRUE;
     }
 
-    if(!(hints_ret->flags&PMinSize)){
+    if(!hints_ret->min_set){
         hints_ret->min_width=0;
         hints_ret->min_height=0;
-        hints_ret->flags|=PMinSize;
+        hints_ret->min_set=TRUE;
     }
     
     hints_ret->base_width+=woff;
@@ -414,6 +513,20 @@ void frame_resize_hints(WFrame *frame, XSizeHints *hints_ret)
     hints_ret->max_height+=hoff;
     hints_ret->min_width+=woff;
     hints_ret->min_height+=hoff;
+    
+    if(frame->barmode==FRAME_BAR_SHAPED){
+        int f=frame->flags&(FRAME_SHADED|FRAME_SHADED_TOGGLE);
+        
+        if(f==FRAME_SHADED || f==FRAME_SHADED_TOGGLE){
+            hints_ret->min_height=frame->bar_h;
+            hints_ret->max_height=frame->bar_h;
+            hints_ret->base_height=frame->bar_h;
+            if(!hints_ret->max_set){
+                hints_ret->max_width=INT_MAX;
+                hints_ret->max_set=TRUE;
+            }
+        }
+    }
 }
 
 
@@ -438,86 +551,109 @@ void frame_activated(WFrame *frame)
 /*}}}*/
 
 
+/*{{{ Client window rqgeom */
+
+
+static void frame_managed_rqgeom_absolute(WFrame *frame, WRegion *sub,
+                                          int rqflags, const WRectangle *geom_,
+                                          WRectangle *geomret)
+{
+    int gravity=ForgetGravity;
+    WRectangle off;
+    WRegion *par;
+    WRectangle geom=*geom_;
+    
+    if(!FORWARD_CWIN_RQGEOM(frame)){
+        region_managed_rqgeom_absolute_default((WRegion*)frame, sub, 
+                                               rqflags, geom_, geomret);
+        return;
+    }
+
+    /* client window gravity */ {
+        WClientWin *cwin=OBJ_CAST(sub, WClientWin);
+        
+        if(cwin!=NULL && cwin->size_hints.flags&PWinGravity)
+            gravity=cwin->size_hints.win_gravity;
+    }
+    
+    mplex_managed_geom(&frame->mplex, &off);
+    off.x=-off.x;
+    off.y=-off.y;
+    off.w=REGION_GEOM(frame).w-off.w;
+    off.h=REGION_GEOM(frame).h-off.h;
+
+    geom.w=maxof(geom.w+off.w, 0);
+    geom.h=maxof(geom.h+off.h, 0);
+    
+    /*region_size_hints_correct((WRegion*)frame, &(geom.w), &(geom.h), TRUE);*/
+    
+    /* If WEAK_? is set, then geom.(x|y) is root-relative as it was not 
+     * requested by the client and clientwin_handle_configure_request has
+     * no better guess. Otherwise the coordinates are those requested by 
+     * the client (modulo borders/gravity) and we interpret them to be 
+     * root-relative coordinates for this frame modulo gravity.
+     */
+    if(rqflags&REGION_RQGEOM_WEAK_X)
+        geom.x+=off.x;
+    else
+        geom.x+=xgravity_deltax(gravity, -off.x, off.x+off.w);
+
+    if(rqflags&REGION_RQGEOM_WEAK_Y)
+        geom.y+=off.y;
+    else
+        geom.y+=xgravity_deltay(gravity, -off.y, off.y+off.h);
+    
+    region_rqgeom((WRegion*)frame, rqflags, &geom, geomret);
+    
+    if(geomret!=NULL){
+        geomret->x-=off.x;
+        geomret->y-=off.y;
+        geomret->w-=off.w;
+        geomret->h-=off.h;
+    }
+}
+
+
+/*}}}*/
+
+
 /*{{{ Misc. */
 
 
-bool frame_set_tabbar(WFrame *frame, int sp)
-{
-    bool set=!(frame->flags&FRAME_TAB_HIDE);
-    bool nset=libtu_do_setparam(sp, set);
-    
-    if(!nset && frame->flags&FRAME_SHADED)
-        return set;
-    
-    if(XOR(nset, set)){
-        frame->flags^=FRAME_TAB_HIDE;
-        mplex_size_changed(&(frame->mplex), FALSE, TRUE);
-        mplex_fit_managed(&(frame->mplex));
-        XClearWindow(ioncore_g.dpy, frame->mplex.win.win);
-        window_draw((WWindow*)frame, TRUE);
-    }
-    
-    return !(frame->flags&FRAME_TAB_HIDE);
-}
-
-
-/*EXTL_DOC
- * Set tab-bar visibility according to the parameter \var{how} 
- * (set/unset/toggle). Resulting state is returned, which may not be
- * what was requested.
- */
-EXTL_EXPORT_AS(WFrame, set_tabbar)
-bool frame_set_tabbar_extl(WFrame *frame, const char *how)
-{
-    return frame_set_tabbar(frame, libtu_string_to_setparam(how));
-}
-
-
-/*EXTL_DOC
- * Is \var{frame}'s tab-bar visible?
- */
-EXTL_SAFE
-EXTL_EXPORT_MEMBER
-bool frame_is_tabbar(WFrame *frame)
-{
-    return (frame->flags&FRAME_TAB_HIDE);
-}
-
-
-static void frame_do_toggle_shade(WFrame *frame, int shaded_h)
-{
-    WRectangle geom=REGION_GEOM(frame);
-
-    if(frame->flags&FRAME_SHADED){
-        if(!(frame->flags&FRAME_SAVED_VERT))
-            return;
-        geom.h=frame->saved_h;
-    }else{
-        if(frame->flags&FRAME_TAB_HIDE)
-            return;
-        geom.h=shaded_h;
-    }
-    
-    region_rqgeom((WRegion*)frame, REGION_RQGEOM_H_ONLY, &geom, NULL);
-}
-
-
 bool frame_set_shaded(WFrame *frame, int sp)
 {
     bool set=(frame->flags&FRAME_SHADED);
     bool nset=libtu_do_setparam(sp, set);
+    WRectangle geom=REGION_GEOM(frame);
     GrBorderWidths bdw;
-    int h=frame->bar_h;
+    int h;
 
     if(!XOR(nset, set))
         return nset;
-    
-    if(!(frame->flags&FRAME_BAR_OUTSIDE) && frame->brush!=NULL){
-        grbrush_get_border_widths(frame->brush, &bdw);
-        h+=bdw.top+bdw.bottom+2*bdw.spacing;
+        
+    if(!nset){
+        if(!(frame->flags&FRAME_SAVED_VERT))
+            return FALSE;
+        geom.h=frame->saved_h;
+    }else{
+        if(frame->barmode==FRAME_BAR_NONE){
+            return FALSE;
+        }else if(frame->barmode==FRAME_BAR_SHAPED){
+            geom.h=frame->bar_h;
+        }else{
+            WRectangle tmp;
+            
+            frame_border_inner_geom(frame, &tmp);
+            
+            geom.h=geom.h-tmp.h;
+        }
     }
-
-    frame_do_toggle_shade((WFrame*)frame, h);
+    
+    frame->flags|=FRAME_SHADED_TOGGLE;
+    
+    region_rqgeom((WRegion*)frame, REGION_RQGEOM_H_ONLY, &geom, NULL);
+    
+    frame->flags&=~FRAME_SHADED_TOGGLE;
     
     return (frame->flags&FRAME_SHADED);
 }
@@ -546,9 +682,44 @@ bool frame_is_shaded(WFrame *frame)
 }
 
 
-void frame_managed_notify(WFrame *frame, WRegion *sub)
+bool frame_set_numbers(WFrame *frame, int sp)
+{
+    bool set=frame->flags&FRAME_SHOW_NUMBERS;
+    bool nset=libtu_do_setparam(sp, set);
+    
+    if(XOR(nset, set)){
+        frame->flags^=FRAME_SHOW_NUMBERS;
+        frame_recalc_bar(frame);
+        frame_draw_bar(frame, TRUE);
+    }
+    
+    return frame->flags&FRAME_SHOW_NUMBERS;
+}
+
+
+/*EXTL_DOC
+ * Control whether tabs show numbers (set/unset/toggle). 
+ * Resulting state is returned, which may not be what was 
+ * requested.
+ */
+EXTL_EXPORT_AS(WFrame, set_numbers)
+bool frame_set_numbers_extl(WFrame *frame, const char *how)
+{
+    return frame_set_numbers(frame, libtu_string_to_setparam(how));
+}
+
+
+/*EXTL_DOC
+ * Does \var{frame} show numbers for tabs?
+ */
+bool frame_is_numbers(WFrame *frame)
+{
+    return frame->flags&FRAME_SHOW_NUMBERS;
+}
+
+
+void frame_managed_notify(WFrame *frame, WRegion *sub, const char *how)
 {
-    /* TODO: Should only draw/update the affected tab.*/
     update_attrs(frame);
     frame_recalc_bar(frame);
     frame_draw_bar(frame, FALSE);
@@ -558,9 +729,15 @@ void frame_managed_notify(WFrame *frame, WRegion *sub)
 static void frame_size_changed_default(WFrame *frame,
                                        bool wchg, bool hchg)
 {
+    int bar_w=frame->bar_w;
+    
     if(wchg)
         frame_recalc_bar(frame);
-    /* We should get a request from X to draw the frame... */
+    
+    if(frame->barmode==FRAME_BAR_SHAPED &&
+       ((!wchg && hchg) || (wchg && bar_w==frame->bar_w))){
+        frame_set_shape(frame);
+    }
 }
 
 
@@ -586,14 +763,14 @@ static void frame_managed_changed(WFrame *frame, int mode, bool sw,
 }
 
 
-#define EMPTY_SHOULD_BE_DESTROYED(FRAME) \
-    ((FRAME)->flags&FRAME_DEST_EMPTY && FRAME_MCOUNT(FRAME)==0 && \
+#define EMPTY_AND_SHOULD_BE_DESTROYED(FRAME) \
+    (DEST_EMPTY(frame) && FRAME_MCOUNT(FRAME)==0 && \
      !OBJ_IS_BEING_DESTROYED(frame))
 
 
 static void frame_destroy_empty(WFrame *frame)
 {
-    if(EMPTY_SHOULD_BE_DESTROYED(frame))
+    if(EMPTY_AND_SHOULD_BE_DESTROYED(frame))
         destroy_obj((Obj*)frame);
 }
 
@@ -601,7 +778,7 @@ static void frame_destroy_empty(WFrame *frame)
 void frame_managed_remove(WFrame *frame, WRegion *reg)
 {
     mplex_managed_remove((WMPlex*)frame, reg);
-    if(EMPTY_SHOULD_BE_DESTROYED(frame)){
+    if(EMPTY_AND_SHOULD_BE_DESTROYED(frame)){
         mainloop_defer_action((Obj*)frame, 
                               (WDeferredAction*)frame_destroy_empty);
     }
@@ -627,19 +804,18 @@ ExtlTab frame_get_configuration(WFrame *frame)
 {
     ExtlTab tab=mplex_get_configuration(&frame->mplex);
     
-    extl_table_sets_i(tab, "flags", frame->flags);
-    
+    extl_table_sets_i(tab, "mode", frame->mode);
+
     if(frame->flags&FRAME_SAVED_VERT){
         extl_table_sets_i(tab, "saved_y", frame->saved_y);
         extl_table_sets_i(tab, "saved_h", frame->saved_h);
     }
+
     if(frame->flags&FRAME_SAVED_HORIZ){
         extl_table_sets_i(tab, "saved_x", frame->saved_x);
         extl_table_sets_i(tab, "saved_w", frame->saved_w);
     }
     
-    extl_table_sets_s(tab, "frame_style", stringstore_get(frame->style));
-    
     return tab;
 }
 
@@ -650,18 +826,6 @@ void frame_do_load(WFrame *frame, ExtlTab tab)
     int flags=0;
     int p=0, s=0;
     
-    extl_table_gets_i(tab, "flags", &flags);
-    
-    if(flags&FRAME_TAB_HIDE){
-        frame->flags&=~FRAME_SHADED;
-        frame->flags|=FRAME_TAB_HIDE;
-    }
-
-    frame->flags|=flags&(FRAME_DEST_EMPTY|
-                         FRAME_MAXED_VERT|
-                         FRAME_MAXED_HORIZ|
-                         FRAME_SZH_USEMINMAX);
-    
     if(extl_table_gets_i(tab, "saved_x", &p) &&
        extl_table_gets_i(tab, "saved_w", &s)){
         frame->saved_x=p;
@@ -682,19 +846,28 @@ void frame_do_load(WFrame *frame, ExtlTab tab)
 
 WRegion *frame_load(WWindow *par, const WFitParams *fp, ExtlTab tab)
 {
-    char *style=NULL;
+    int mode=FRAME_MODE_UNKNOWN;
     WFrame *frame;
     
-    extl_table_gets_s(tab, "frame_style", &style);
+    if(!extl_table_gets_i(tab, "mode", &mode)){
+        #warning "TODO: Remove backwards compatibility hack"
+        char *style=NULL;
+        if(extl_table_gets_s(tab, "frame_style", &style)){
+            if(strcmp(style, "frame-tiled")==0)
+                mode=FRAME_MODE_TILED;
+            else if(strcmp(style, "frame-floating")==0)
+                mode=FRAME_MODE_FLOATING;
+            else if(strcmp(style, "frame-transientcontainer")==0)
+                mode=FRAME_MODE_TRANSIENT;
+            free(style);
+        }
+    }
     
-    frame=create_frame(par, fp, style);
+    frame=create_frame(par, fp, mode);
     
     if(frame!=NULL)
         frame_do_load(frame, tab);
     
-    if(style!=NULL)
-        free(style);
-    
     return (WRegion*)frame;
 }
 
@@ -706,7 +879,7 @@ WRegion *frame_load(WWindow *par, const WFitParams *fp, ExtlTab tab)
 
 
 static DynFunTab frame_dynfuntab[]={
-    {region_size_hints, frame_resize_hints},
+    {region_size_hints, frame_size_hints},
 
     {mplex_managed_changed, frame_managed_changed},
     {mplex_size_changed, frame_size_changed_default},
@@ -721,19 +894,10 @@ static DynFunTab frame_dynfuntab[]={
      (DynFun*)frame_get_configuration},
 
     {window_draw, 
-     frame_draw_default},
-    {frame_draw_bar, 
-     frame_draw_bar_default},
-    {frame_recalc_bar, 
-     frame_recalc_bar_default},
-    {frame_bar_geom, 
-     frame_bar_geom_default},
-    {frame_border_geom, 
-     frame_border_geom_default},
-    {frame_border_inner_geom, 
-     frame_border_inner_geom_default},
+     frame_draw},
+    
     {mplex_managed_geom, 
-     frame_managed_geom_default},
+     frame_managed_geom},
 
     {region_updategr, 
      frame_updategr},
@@ -741,14 +905,14 @@ static DynFunTab frame_dynfuntab[]={
     {(DynFun*)region_fitrep,
      (DynFun*)frame_fitrep},
 
+    {region_managed_rqgeom_absolute, 
+     frame_managed_rqgeom_absolute},
+
     {region_managed_remove, frame_managed_remove},
     
     {(DynFun*)mplex_default_index,
      (DynFun*)frame_default_index},
     
-    {frame_brushes_updated, 
-     frame_brushes_updated_default},
-    
     END_DYNFUNTAB
 };
                                        
