@@ -1,7 +1,7 @@
 /*
  * ion/ioncore/fullscreen.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2005. 
+ * Copyright (c) Tuomo Valkonen 1999-2006. 
  *
  * Ion is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by
@@ -17,7 +17,7 @@
 #include "screen.h"
 #include "clientwin.h"
 
-#define CLIENTWIN_IS_FULLSCREEN(cwin) OBJ_IS(REGION_PARENT(cwin), WScreen)
+#define REGION_IS_FULLSCREEN(REG) OBJ_IS(REGION_PARENT(REG), WScreen)
 
 extern bool clientwin_check_fullscreen_request(WClientWin *cwin, 
                                                int w, int h, bool switchto);
