@@ -53,6 +53,9 @@
 #include "saveload.h"
 #include "infowin.h"
 #include "activity.h"
+#include "group-cw.h"
+#include "group-ws.h"
+
 #include "../version.h"
 #include "exports.h"
 
@@ -268,11 +271,12 @@ static bool init_hooks()
     INIT_HOOK_(clientwin_mapped_hook);
     INIT_HOOK_(clientwin_unmapped_hook);
     INIT_HOOK_(clientwin_property_change_hook);
-    INIT_HOOK_(region_activity_hook);
+    INIT_HOOK_(region_notify_hook);
+    
     INIT_HOOK(clientwin_do_manage_alt, clientwin_do_manage_default);
     INIT_HOOK(ioncore_handle_event_alt, ioncore_handle_event);
     INIT_HOOK(region_do_warp_alt, region_do_warp_default);
-    
+
     mainloop_sigchld_hook=mainloop_register_hook("ioncore_sigchld_hook",
                                                  create_hook());
     mainloop_sigusr2_hook=mainloop_register_hook("ioncore_sigusr2_hook",
@@ -294,6 +298,10 @@ static bool register_classes()
                                      (WRegionLoadCreateFn*)frame_load);
     fail|=!ioncore_register_regclass(&CLASSDESCR(WInfoWin), 
                                      (WRegionLoadCreateFn*)infowin_load);
+    fail|=!ioncore_register_regclass(&CLASSDESCR(WGroupCW), 
+                                     (WRegionLoadCreateFn*)groupcw_load);
+    fail|=!ioncore_register_regclass(&CLASSDESCR(WGroupWS), 
+                                     (WRegionLoadCreateFn*)groupws_load);
     
     return !fail;
 }
