@@ -1,7 +1,7 @@
 /*
  * ion/ioncore/netwm.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2005. 
+ * Copyright (c) Tuomo Valkonen 1999-2006. 
  *
  * Ion is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by
@@ -116,7 +116,7 @@ void netwm_update_state(WClientWin *cwin)
     CARD32 data[1];
     int n=0;
     
-    if(CLIENTWIN_IS_FULLSCREEN(cwin))
+    if(REGION_IS_FULLSCREEN(cwin))
         data[n++]=atom_net_wm_state_fullscreen;
 
     XChangeProperty(ioncore_g.dpy, cwin->win, atom_net_wm_state, 
@@ -142,20 +142,24 @@ static void netwm_state_change_rq(WClientWin *cwin,
     }
     
     /* Ok, full screen add/remove/toggle */
-    if(!CLIENTWIN_IS_FULLSCREEN(cwin)){
+    if(!REGION_IS_FULLSCREEN(cwin)){
         if(ev->data.l[0]==_NET_WM_STATE_ADD || 
            ev->data.l[0]==_NET_WM_STATE_TOGGLE){
             bool sw=clientwin_fullscreen_may_switchto(cwin);
             cwin->flags|=CLIENTWIN_FS_RQ;
             if(!clientwin_enter_fullscreen(cwin, sw))
                 cwin->flags&=~CLIENTWIN_FS_RQ;
+        }else{
+            /* Should not be set.. */
+            cwin->flags&=~CLIENTWIN_FS_RQ;
         }
     }else{
         if(ev->data.l[0]==_NET_WM_STATE_REMOVE || 
            ev->data.l[0]==_NET_WM_STATE_TOGGLE){
             bool sw=clientwin_fullscreen_may_switchto(cwin);
+            cwin->flags&=~CLIENTWIN_FS_RQ;
             clientwin_leave_fullscreen(cwin, sw);
-        }else if(CLIENTWIN_IS_FULLSCREEN(cwin)){
+        }else{
             /* Set the flag */
             cwin->flags|=CLIENTWIN_FS_RQ;
         }
