@@ -10,11 +10,12 @@
  */
 
 #include <string.h>
+#include <libtu/objp.h>
+#include <libmainloop/defer.h>
 #include "common.h"
 #include "region.h"
 #include "binding.h"
 #include "regbind.h"
-#include <libtu/objp.h>
 
 
 /*{{{ Grab/ungrab */
@@ -97,12 +98,13 @@ void rbind_binding_removed(const WRegBindingInfo *rbind,
 /*{{{ Find */
 
 
-static WRegBindingInfo *find_rbind(WRegion *reg, WBindmap *bindmap)
+static WRegBindingInfo *find_rbind(WRegion *reg, WBindmap *bindmap,
+                                   WRegion *owner)
 {
     WRegBindingInfo *rbind;
     
     for(rbind=(WRegBindingInfo*)reg->bindings; rbind!=NULL; rbind=rbind->next){
-        if(rbind->bindmap==bindmap)
+        if(rbind->bindmap==bindmap && rbind->owner==owner)
             return rbind;
     }
     
@@ -116,34 +118,34 @@ static WRegBindingInfo *find_rbind(WRegion *reg, WBindmap *bindmap)
 /*{{{ Interface */
 
 
-bool region_add_bindmap_owned(WRegion *reg, WBindmap *bindmap, WRegion *owner)
+static WRegBindingInfo *region_do_add_bindmap_owned(WRegion *reg, 
+                                                    WBindmap *bindmap, 
+                                                    WRegion *owner,
+                                                    bool first)
 {
     WRegBindingInfo *rbind;
     
-    if(region_xwindow(reg)==None)
-        return FALSE;
-    
     if(bindmap==NULL)
-        return FALSE;
-    
-    if(find_rbind(reg, bindmap)!=NULL)
-        return FALSE;
+        return NULL;
     
     rbind=ALLOC(WRegBindingInfo);
     
     if(rbind==NULL)
-        return FALSE;
+        return NULL;
     
     rbind->bindmap=bindmap;
     rbind->owner=owner;
     rbind->reg=reg;
+    rbind->tmp=0;
+    
     LINK_ITEM(bindmap->rbind_list, rbind, bm_next, bm_prev);
 
-    grab_ungrabbed_bindings(reg, bindmap);
+    if(region_xwindow(reg)!=None && !(reg->flags&REGION_GRAB_ON_PARENT))
+        grab_ungrabbed_bindings(reg, bindmap);
     
     /* Link to reg's rbind list*/ {
         WRegBindingInfo *b=reg->bindings;
-        if(owner==NULL){
+        if(first){
             LINK_ITEM_FIRST(b, rbind, next, prev);
         }else{
             LINK_ITEM_LAST(b, rbind, next, prev);
@@ -151,13 +153,15 @@ bool region_add_bindmap_owned(WRegion *reg, WBindmap *bindmap, WRegion *owner)
         reg->bindings=b;
     }
     
-    return TRUE;
+    return rbind;
 }
 
 
 bool region_add_bindmap(WRegion *reg, WBindmap *bindmap)
 {
-    return region_add_bindmap_owned(reg, bindmap, NULL);
+    if(find_rbind(reg, bindmap, NULL)!=NULL)
+        return FALSE;
+    return (region_do_add_bindmap_owned(reg, bindmap, NULL, TRUE)!=NULL);
 }
 
 
@@ -170,26 +174,19 @@ static void remove_rbind(WRegion *reg, WRegBindingInfo *rbind)
         UNLINK_ITEM(b, rbind, next, prev);
         reg->bindings=b;
     }
-
-    ungrab_freed_bindings(reg, rbind->bindmap);
+    
+    if(region_xwindow(reg)!=None && !(reg->flags&REGION_GRAB_ON_PARENT))
+        ungrab_freed_bindings(reg, rbind->bindmap);
 
     free(rbind);
 }
 
 
-void region_remove_bindmap_owned(WRegion *reg, WBindmap *bindmap,
-                                 WRegion *owner)
-{
-    WRegBindingInfo *rbind=find_rbind(reg, bindmap);
-    
-    if(rbind!=NULL /*&& rbind->owner==owner*/)
-        remove_rbind(reg, rbind);
-}
-
-
 void region_remove_bindmap(WRegion *reg, WBindmap *bindmap)
 {
-    region_remove_bindmap_owned(reg, bindmap, NULL);
+    WRegBindingInfo *rbind=find_rbind(reg, bindmap, NULL);
+    if(rbind!=NULL)
+        remove_rbind(reg, rbind);
 }
 
 
@@ -213,7 +210,7 @@ WBinding *region_lookup_keybinding(WRegion *reg, const XKeyEvent *ev,
     int i;
     
     *binding_owner_ret=reg;
-    
+
     for(rbind=(WRegBindingInfo*)reg->bindings; rbind!=NULL; rbind=rbind->next){
         bindmap=rbind->bindmap;
         
@@ -250,7 +247,7 @@ WBinding *region_lookup_keybinding(WRegion *reg, const XKeyEvent *ev,
 
 
 WBinding *region_lookup_binding(WRegion *reg, int act, uint state,
-                                     uint kcb, int area)
+                                uint kcb, int area)
 {
     WRegBindingInfo *rbind;
     WBinding *binding=NULL;
@@ -269,3 +266,72 @@ WBinding *region_lookup_binding(WRegion *reg, int act, uint state,
 
 /*}}}*/
 
+
+/*{{{ Update */
+
+
+static void add_bindings(WRegion *reg, WRegion *r2)
+{
+    WRegion *rx=REGION_MANAGER(r2);
+    WRegBindingInfo *rbind, *rb2;
+    WBinding *binding=NULL;
+    
+    if(rx!=NULL && REGION_PARENT_REG(rx)==reg){
+        /* The recursion is here to get the bindmaps correctly ordered. */
+        add_bindings(reg, rx);
+    }
+    
+    if(r2->flags&REGION_GRAB_ON_PARENT){
+        for(rb2=(WRegBindingInfo*)r2->bindings; rb2!=NULL; rb2=rb2->next){
+            rbind=find_rbind(reg, rb2->bindmap, r2);
+            if(rbind==NULL){
+                rbind=region_do_add_bindmap_owned(reg, rb2->bindmap, 
+                                                  r2, TRUE);
+            }
+            if(rbind!=NULL)
+                rbind->tmp=1;
+        }
+    }
+}
+
+
+void region_do_update_owned_grabs(WRegion *reg)
+{
+    WRegBindingInfo *rbind, *rb2;
+
+    reg->flags&=~REGION_BINDING_UPDATE_SCHEDULED;
+    
+    /* clear flags */
+    for(rbind=(WRegBindingInfo*)reg->bindings; rbind!=NULL; rbind=rbind->next)
+        rbind->tmp=0;
+    
+    /* make new grabs */
+    if(reg->active_sub!=NULL)
+        add_bindings(reg, reg->active_sub);
+    
+    /* remove old grabs */
+    for(rbind=(WRegBindingInfo*)reg->bindings; rbind!=NULL; rbind=rb2){
+        rb2=rbind->next;
+        if(rbind->tmp!=1 && rbind->owner!=NULL)
+            remove_rbind(reg, rbind);
+    }
+}
+
+void region_update_owned_grabs(WRegion *reg)
+{
+    if(reg->flags&REGION_BINDING_UPDATE_SCHEDULED 
+       || OBJ_IS_BEING_DESTROYED(reg)
+       || ioncore_g.opmode==IONCORE_OPMODE_DEINIT){
+        return;
+    }
+    
+    if(mainloop_defer_action((Obj*)reg, 
+                             (WDeferredAction*)region_do_update_owned_grabs)){
+        reg->flags|=REGION_BINDING_UPDATE_SCHEDULED;
+    }else{
+        region_do_update_owned_grabs(reg);
+    }
+}
+
+
+/*}}}*/
