@@ -18,12 +18,15 @@
 #include "binding.h"
 
 
+DECLSTRUCT(WSubmapState){
+    uint key, state;
+    WSubmapState *next;
+};
+
+
 extern bool region_add_bindmap(WRegion *reg, WBindmap *bindmap);
-extern bool region_add_bindmap_owned(WRegion *reg, WBindmap *bindmap,
-                                     WRegion *owner);
 extern void region_remove_bindmap(WRegion *reg, WBindmap *bindmap);
-extern void region_remove_bindmap_owned(WRegion *reg, WBindmap *bindmap,
-                                        WRegion *owner);
+
 extern void region_remove_bindings(WRegion *reg);
 
 extern WBinding *region_lookup_keybinding(WRegion *reg, const XKeyEvent *ev,
@@ -39,5 +42,8 @@ extern void rbind_binding_removed(const WRegBindingInfo *rbind,
                                   const WBinding *binding,
                                   const WBindmap *bindmap);
 
+extern void region_update_owned_grabs(WRegion *reg);
+extern void region_do_update_owned_grabs(WRegion *reg);
+
 #endif /* ION_IONCORE_REGBIND_H */
 
