@@ -9,6 +9,8 @@
  * (at your option) any later version.
  */
 
+#include <string.h>
+
 #include <libtu/objp.h>
 #include <libextl/extl.h>
 #include <libmainloop/defer.h>
@@ -29,6 +31,12 @@
 #define D2(X)
 
 
+WHook *region_notify_hook=NULL;
+
+static void region_notify_change_(WRegion *reg, const char *how, 
+                                  Obj *detail);
+
+
 /*{{{ Init & deinit */
 
 
@@ -57,6 +65,8 @@ void region_init(WRegion *reg, WWindow *par, const WFitParams *fp)
     
     reg->manager=NULL;
     
+    reg->submapstat=NULL;
+    
     reg->mgd_activity=FALSE;
 
     if(par!=NULL){
@@ -194,29 +204,33 @@ void region_managed_inactivated(WRegion *mgr, WRegion *reg)
 }
 
 
-static bool region_managed_goto_default(WRegion *mgr, WRegion *reg, int flags)
+static bool region_managed_prepare_focus_default(WRegion *mgr, WRegion *reg, 
+                                                 int flags, 
+                                                 WPrepareFocusResult *res)
 {
-    if(!region_is_fully_mapped(mgr))
+    if(!region_prepare_focus(mgr, flags, res))
         return FALSE;
-    if(!REGION_IS_MAPPED(reg))
-        region_map(reg);
-    if(flags&REGION_GOTO_FOCUS)
-        region_maybewarp(reg, !(flags&REGION_GOTO_NOWARP));
+    
+    res->reg=reg;
+    res->flags=flags;
     return TRUE;
 }
 
     
-bool region_managed_goto(WRegion *mgr, WRegion *reg, int flags)
+bool region_managed_prepare_focus(WRegion *mgr, WRegion *reg, 
+                                  int flags, 
+                                  WPrepareFocusResult *res)
 {
     bool ret=TRUE;
-    CALL_DYN_RET(ret, bool, region_managed_goto, mgr, (mgr, reg, flags));
+    CALL_DYN_RET(ret, bool, region_managed_prepare_focus, mgr, 
+                 (mgr, reg, flags, res));
     return ret;
 }
 
 
-void region_managed_notify(WRegion *mgr, WRegion *reg)
+void region_managed_notify(WRegion *mgr, WRegion *reg, const char *how)
 {
-    CALL_DYN(region_managed_notify, mgr, (mgr, reg));
+    CALL_DYN(region_managed_notify, mgr, (mgr, reg, how));
 }
 
 
@@ -246,15 +260,6 @@ void region_child_removed(WRegion *reg, WRegion *sub)
 }
 
 
-void region_manager_changed(WRegion *reg, WRegion *mgr_or_null)
-{
-    CALL_DYN(region_manager_changed, reg, (reg, mgr_or_null));
-}
-
-
-/*}}}*/
-
-
 /*}}}*/
 
 
@@ -274,34 +279,55 @@ void region_updategr_default(WRegion *reg)
 /*}}}*/
 
 
+/*}}}*/
+
 
 /*{{{ Goto */
 
 
-bool region_goto_flags(WRegion *reg, int flags)
+bool region_prepare_focus(WRegion *reg, int flags, 
+                          WPrepareFocusResult *res)
 {
     WRegion *mgr=REGION_MANAGER(reg);
-    
-    if(mgr!=NULL){
-        if(!region_goto_flags(mgr, flags))
-            return FALSE;
-        return region_managed_goto(mgr, reg, flags);
-    }else{
-        WRegion *par=REGION_PARENT_REG(reg);
-        if(par!=NULL){
-            if(!region_goto_flags(par, flags))
-                return FALSE;
-        }
+    WRegion *par=REGION_PARENT_REG(reg);
 
-        region_map(reg);
-        if(flags&REGION_GOTO_FOCUS)
-            region_maybewarp(reg, !(flags&REGION_GOTO_NOWARP));
-        
+    if(REGION_IS_MAPPED(reg) && region_may_control_focus(reg)){
+        res->reg=reg;
+        res->flags=0;
+        return TRUE;
+    }else{
+        if(mgr!=NULL){
+            return region_managed_prepare_focus(mgr, reg, flags, res);
+        }else if(par!=NULL){
+            if(!region_prepare_focus(par, flags, res))
+                return FALSE;
+            /* Just focus reg, if it has no manager, and parent can be 
+             * focused. 
+             */
+        }
+        res->reg=reg;
+        res->flags=flags;
         return TRUE;
     }
 }
 
 
+bool region_goto_flags(WRegion *reg, int flags)
+{
+    WPrepareFocusResult res;
+    bool ret;
+    
+    ret=region_prepare_focus(reg, flags, &res);
+    
+    if(res.reg!=NULL){
+        if(res.flags&REGION_GOTO_FOCUS)
+            region_maybewarp(res.reg, !(res.flags&REGION_GOTO_NOWARP));
+    }
+    
+    return ret;
+}
+
+
 /*EXTL_DOC
  * Attempt to display \var{reg}, save region activity status and then
  * warp to (or simply set focus to if warping is disabled) \var{reg}.
@@ -492,25 +518,38 @@ void region_detach_manager(WRegion *reg)
 }
 
 
-/* This should only be called within region_managed_remove */
+/* This should only be called within region_managed_remove,
+ * _after_ any managed lists and other essential structures
+ * of mgr have been broken.
+ */
 void region_unset_manager(WRegion *reg, WRegion *mgr)
 {
     if(reg->manager!=mgr)
         return;
+    
     reg->manager=NULL;
     
-    if(!OBJ_IS_BEING_DESTROYED(reg))
-        region_manager_changed(reg, NULL);
+    if(region_is_activity_r(reg))
+        region_clear_mgd_activity(mgr);
+
+    region_notify_change_(reg, "unset_manager", (Obj*)mgr);
 }
 
 
+/* This should be called within region attach routines,
+ * _after_ any managed lists and other essential structures
+ * of mgr have been set up.
+ */
 void region_set_manager(WRegion *reg, WRegion *mgr)
 {
     assert(reg->manager==NULL);
     
     reg->manager=mgr;
     
-    region_manager_changed(reg, mgr);
+    if(region_is_activity_r(reg))
+        region_mark_mgd_activity(mgr);
+    
+    region_notify_change_(reg, "set_manager", (Obj*)mgr);
 }
 
 
@@ -532,8 +571,10 @@ void region_unset_parent(WRegion *reg)
     UNLINK_ITEM(p->children, reg, p_next, p_prev);
     reg->parent=NULL;
 
-    if(p->active_sub==reg)
+    if(p->active_sub==reg){
         p->active_sub=NULL;
+        region_update_owned_grabs(p);
+    }
     
     region_child_removed(p, reg);
 }
@@ -586,19 +627,7 @@ WRegion *region_get_manager_chk(WRegion *p, const ClassDescr *descr)
 /*}}}*/
 
 
-/*{{{ Stacking */
-
-
-void region_raise(WRegion *reg)
-{
-    region_restack(reg, None, Above);
-}
-
-
-void region_lower(WRegion *reg)
-{
-    region_restack(reg, None, Below);
-}
+/*{{{ Stacking and ordering */
 
 
 static void region_stacking_default(WRegion *reg, 
@@ -622,6 +651,47 @@ void region_restack(WRegion *reg, Window other, int mode)
 }
 
 
+
+bool region_managed_rqorder(WRegion *reg, WRegion *sub, WRegionOrder order)
+{
+    bool ret=FALSE;
+    CALL_DYN_RET(ret, bool, region_managed_rqorder, reg, (reg, sub, order));
+    return ret;
+}
+
+
+bool region_rqorder(WRegion *reg, WRegionOrder order)
+{
+    WRegion *mgr=REGION_MANAGER(reg);
+    
+    if(mgr==NULL)
+        return FALSE;
+    else
+        return region_managed_rqorder(mgr, reg, order);
+}
+
+
+/*EXTL_DOC
+ * Request ordering. Currently supported values for \var{ord}
+ * are 'front' and 'back'.
+ */
+EXTL_EXPORT_AS(WRegion, rqorder)
+bool region_rqorder_extl(WRegion *reg, const char *ord)
+{
+    WRegionOrder order;
+    
+    if(strcmp(ord, "front")==0){
+        order=REGION_ORDER_FRONT;
+    }else if(strcmp(ord, "back")==0){
+        order=REGION_ORDER_BACK;
+    }else{
+        return FALSE;
+    }
+    
+    return region_rqorder(reg, order);
+}
+
+
 /*}}}*/
 
 
@@ -706,12 +776,54 @@ void region_rootpos(WRegion *reg, int *xret, int *yret)
 }
 
 
-void region_notify_change(WRegion *reg)
+static bool mrsh_not(WHookDummy *fn, void *p)
+{
+    WRegion *reg=(WRegion*)((void**)p)[0];
+    const char *how=(const char*)((void**)p)[1];
+    Obj *detail=(Obj*)((void**)p)[2];
+    
+    fn(reg, how, detail);
+    
+    return TRUE;
+}
+
+
+static bool mrshe_not(ExtlFn fn, void *p)
+{
+    WRegion *reg=(WRegion*)((void**)p)[0];
+    const char *how=(const char*)((void**)p)[1];
+    Obj *detail=(Obj*)((void**)p)[2];
+    
+    extl_call(fn, "oso", NULL, reg, how, detail);
+    
+    return TRUE;
+}
+
+
+static void region_notify_change_(WRegion *reg, const char *how, 
+                                  Obj *detail)
+{
+    const void *p[3];
+    
+    p[0]=reg;
+    p[1]=how;
+    p[2]=detail;
+
+    extl_protect(NULL);
+    hook_call(region_notify_hook, p, mrsh_not, mrshe_not),
+    extl_unprotect(NULL);
+    
+}
+
+
+void region_notify_change(WRegion *reg, const char *how)
 {
     WRegion *mgr=REGION_MANAGER(reg);
-    
+
     if(mgr!=NULL)
-        region_managed_notify(mgr, reg);
+        region_managed_notify(mgr, reg, how);
+    
+    region_notify_change_(reg, how, NULL);
 }
 
 
@@ -735,6 +847,21 @@ bool region_handle_drop(WRegion *reg, int x, int y, WRegion *dropped)
 }
 
 
+WRegion *region_managed_within(WRegion *reg, WRegion *mgd)
+{
+    while(mgd!=NULL && 
+          (REGION_PARENT_REG(mgd)==reg ||
+           REGION_PARENT_REG(mgd)==REGION_PARENT_REG(reg))){
+        
+        if(REGION_MANAGER(mgd)==reg)
+            return mgd;
+        mgd=REGION_MANAGER(mgd);
+    }
+    
+    return NULL;
+}
+
+
 /*}}}*/
 
 
@@ -744,6 +871,9 @@ bool region_handle_drop(WRegion *reg, int x, int y, WRegion *dropped)
 static DynFunTab region_dynfuntab[]={
     {region_managed_rqgeom,
      region_managed_rqgeom_allow},
+     
+    {region_managed_rqgeom_absolute,
+     region_managed_rqgeom_absolute_default},
     
     {region_updategr, 
      region_updategr_default},
@@ -757,8 +887,8 @@ static DynFunTab region_dynfuntab[]={
     {(DynFun*)region_prepare_manage_transient,
      (DynFun*)region_prepare_manage_transient_default},
 
-    {(DynFun*)region_managed_goto,
-     (DynFun*)region_managed_goto_default},
+    {(DynFun*)region_managed_prepare_focus,
+     (DynFun*)region_managed_prepare_focus_default},
 
     {(DynFun*)region_rqclose_propagate,
      (DynFun*)region_rqclose_propagate_default},
@@ -766,6 +896,9 @@ static DynFunTab region_dynfuntab[]={
     {(DynFun*)region_rqclose,
      (DynFun*)region_rqclose_default},
     
+    {(DynFun*)region_displayname,
+     (DynFun*)region_name},
+    
     {region_stacking, 
      region_stacking_default},
     
