@@ -142,9 +142,9 @@ static void moveres_draw_infowin(WMoveresMode *mode)
         w=mode->geom.w;
         h=mode->geom.h;
         
-        if((mode->hints.flags&PResizeInc) &&
+        if((mode->hints.inc_set) &&
            (mode->hints.width_inc>1 || mode->hints.height_inc>1)){
-            if(mode->hints.flags&PBaseSize){
+            if(mode->hints.base_set){
                 w-=mode->hints.base_width;
                 h-=mode->hints.base_height;
             }
@@ -258,9 +258,9 @@ static bool moveresmode_init(WMoveresMode *mode, WRegion *reg,
         }
     }
     
-    if(!mode->hints.flags&PMinSize || mode->hints.min_width<1)
+    if(!mode->hints.min_set || mode->hints.min_width<1)
         mode->hints.min_width=1;
-    if(!mode->hints.flags&PMinSize || mode->hints.min_height<1)
+    if(!mode->hints.min_set || mode->hints.min_height<1)
         mode->hints.min_height=1;
     
     /* Set up info window */
@@ -354,7 +354,7 @@ static void moveresmode_delta(WMoveresMode *mode,
     if(h<=0)
         h=mode->hints.min_height;
     
-    xsizehints_correct(&mode->hints, &w, &h, TRUE);
+    sizehints_correct(&mode->hints, &w, &h, TRUE, TRUE);
     
     /* Do not modify coordinates and sizes that were not requested to be
      * changed. 
@@ -491,20 +491,27 @@ bool moveresmode_do_end(WMoveresMode *mode, bool apply)
 void region_rqgeom(WRegion *reg, int flags, const WRectangle *geom,
                    WRectangle *geomret)
 {
-    bool tryonly=(flags&REGION_RQGEOM_TRYONLY);
+    WRegion *mgr=REGION_MANAGER(reg);
     
-    if(REGION_MANAGER(reg)!=NULL){
-        region_managed_rqgeom(REGION_MANAGER(reg), reg, flags, geom,
-                              geomret);
+    if(mgr!=NULL){
+        if(flags&REGION_RQGEOM_ABSOLUTE)
+            region_managed_rqgeom_absolute(mgr, reg, flags, geom, geomret);
+        else
+            region_managed_rqgeom(mgr, reg, flags, geom, geomret);
     }else{
+        WRectangle tmp;
+        
+        if(flags&REGION_RQGEOM_ABSOLUTE){
+            region_absolute_geom_to_parent(reg, geom, &tmp);
+            geom=&tmp;
+        }
+        
         if(geomret!=NULL)
             *geomret=REGION_GEOM(reg);
-        if(!tryonly)
+        
+        if(!(flags&REGION_RQGEOM_TRYONLY))
             region_fit(reg, geom, REGION_FIT_EXACT);
     }
-
-    /*if(!tryonly && geomret!=NULL)
-        *geomret=REGION_GEOM(reg);*/
 }
 
 
@@ -521,6 +528,7 @@ ExtlTab region_rqgeom_extl(WRegion *reg, ExtlTab g)
     WRectangle ogeom=REGION_GEOM(reg);
     int flags=REGION_RQGEOM_WEAK_ALL;
     
+    
     if(extl_table_gets_i(g, "x", &(geom.x)))
        flags&=~REGION_RQGEOM_WEAK_X;
     if(extl_table_gets_i(g, "y", &(geom.y)))
@@ -548,10 +556,12 @@ void region_managed_rqgeom(WRegion *mgr, WRegion *reg,
 }
 
 
-void region_rqgeom_clientwin(WRegion *mgr, WClientWin *cwin,
-                             int flags, const WRectangle *geom)
+void region_managed_rqgeom_absolute(WRegion *mgr, WRegion *reg,
+                                    int flags, const WRectangle *geom,
+                                    WRectangle *geomret)
 {
-    CALL_DYN(region_rqgeom_clientwin, mgr, (mgr, cwin, flags, geom));
+    CALL_DYN(region_managed_rqgeom_absolute, mgr,
+             (mgr, reg, flags, geom, geomret));
 }
 
 
@@ -576,25 +586,51 @@ void region_managed_rqgeom_unallow(WRegion *mgr, WRegion *reg,
 }
 
 
-void region_size_hints(WRegion *reg, XSizeHints *hints_ret)
+void region_managed_rqgeom_absolute_default(WRegion *mgr, WRegion *reg,
+                                            int flags, 
+                                            const WRectangle *geom,
+                                            WRectangle *geomret)
 {
-    hints_ret->flags=0;
+    WRectangle geom2;
+    
+    region_absolute_geom_to_parent(reg, geom, &geom2);
+    
+    region_managed_rqgeom(mgr, reg, flags&~REGION_RQGEOM_ABSOLUTE,
+                          &geom2, geomret);
+}
+
+
+void region_size_hints(WRegion *reg, WSizeHints *hints_ret)
+{
+    sizehints_clear(hints_ret);
+    
     {
         CALL_DYN(region_size_hints, reg, (reg, hints_ret));
     }
-    if(!(hints_ret->flags&PMinSize)){
+    
+    if(!hints_ret->min_set){
         hints_ret->min_width=1;
         hints_ret->min_height=1;
     }
-    if(!(hints_ret->flags&PBaseSize)){
+    if(!hints_ret->base_set){
         hints_ret->base_width=0;
         hints_ret->base_height=0;
     }
-    if(!(hints_ret->flags&PMaxSize)){
+    if(!hints_ret->max_set){
         hints_ret->max_width=INT_MAX;
         hints_ret->max_height=INT_MAX;
     }
-    /*hints_ret->flags|=(PMinSize|PBaseSize|PMaxSize);*/
+}
+
+
+void region_size_hints_correct(WRegion *reg, 
+                               int *wp, int *hp, bool min)
+{
+    WSizeHints hints;
+    
+    region_size_hints(reg, &hints);
+    
+    sizehints_correct(&hints, wp, hp, min, FALSE);
 }
 
 
@@ -617,28 +653,26 @@ EXTL_SAFE
 EXTL_EXPORT_AS(WRegion, size_hints)
 ExtlTab region_size_hints_extl(WRegion *reg)
 {
-    XSizeHints hints;
+    WSizeHints hints;
     ExtlTab tab;
     
     region_size_hints(reg, &hints);
     
     tab=extl_create_table();
     
-    /* Base size is always guaranteed to be set. */
-    extl_table_sets_i(tab, "base_w", hints.base_width);
-    extl_table_sets_i(tab, "base_h", hints.base_height);
-    /* Minimum size is always guaranteed to be set. */
-    extl_table_sets_i(tab, "min_w", hints.min_width);
-    extl_table_sets_i(tab, "min_h", hints.min_height);
-    if(hints.flags&PMaxSize){
-        extl_table_sets_i(tab, "max_w", hints.max_width);
-        extl_table_sets_i(tab, "max_h", hints.max_height);
-    }
-    if(hints.flags&PBaseSize){
+    if(hints.base_set){
         extl_table_sets_i(tab, "base_w", hints.base_width);
         extl_table_sets_i(tab, "base_h", hints.base_height);
     }
-    if(hints.flags&PResizeInc){
+    if(hints.min_set){
+        extl_table_sets_i(tab, "min_w", hints.min_width);
+        extl_table_sets_i(tab, "min_h", hints.min_height);
+    }
+    if(hints.max_set){
+        extl_table_sets_i(tab, "max_w", hints.max_width);
+        extl_table_sets_i(tab, "max_h", hints.max_height);
+    }
+    if(hints.inc_set){
         extl_table_sets_i(tab, "inc_w", hints.width_inc);
         extl_table_sets_i(tab, "inc_h", hints.height_inc);
     }
@@ -652,16 +686,6 @@ ExtlTab region_size_hints_extl(WRegion *reg)
 /*{{{ Restore size, maximize, shade */
 
 
-static void correct_frame_size(WFrame *frame, int *w, int *h)
-{
-    XSizeHints hints;
-    int wdiff, hdiff;
-    
-    region_size_hints((WRegion*)frame, &hints);
-    xsizehints_correct(&hints, w, h, TRUE);
-}
-
-
 static bool rqh(WFrame *frame, int y, int h)
 {
     WRectangle geom, rgeom;
@@ -673,7 +697,7 @@ static bool rqh(WFrame *frame, int y, int h)
     geom.h=h;
     
     dummy_w=geom.w;
-    correct_frame_size(frame, &dummy_w, &(geom.h));
+    region_size_hints_correct((WRegion*)frame,  &dummy_w, &(geom.h), TRUE);
     
     region_rqgeom((WRegion*)frame, REGION_RQGEOM_VERT_ONLY, &geom, &rgeom);
     
@@ -722,7 +746,7 @@ static bool rqw(WFrame *frame, int x, int w)
     geom.h=REGION_GEOM(frame).h;
     
     dummy_h=geom.h;
-    correct_frame_size(frame, &(geom.w), &dummy_h);
+    region_size_hints_correct((WRegion*)frame,  &(geom.w), &dummy_h, TRUE);
     
     region_rqgeom((WRegion*)frame, REGION_RQGEOM_HORIZ_ONLY, &geom, &rgeom);
     
@@ -770,7 +794,7 @@ void frame_maximize_horiz(WFrame *frame)
 
 uint region_min_h(WRegion *reg)
 {
-    XSizeHints hints;
+    WSizeHints hints;
     region_size_hints(reg, &hints);
     return hints.min_height;
 }
@@ -778,7 +802,7 @@ uint region_min_h(WRegion *reg)
 
 uint region_min_w(WRegion *reg)
 {
-    XSizeHints hints;
+    WSizeHints hints;
     region_size_hints(reg, &hints);
     return hints.min_width;
 }
@@ -794,5 +818,24 @@ void region_convert_root_geom(WRegion *reg, WRectangle *geom)
     }
 }
 
+
+void region_absolute_geom_to_parent(WRegion *reg, const WRectangle *rgeom,
+                                    WRectangle *pgeom)
+{
+    WRegion *parent=REGION_PARENT_REG(reg);
+    
+    pgeom->w=rgeom->w;
+    pgeom->h=rgeom->h;
+    
+    if(parent==NULL){
+        pgeom->x=rgeom->x;
+        pgeom->y=rgeom->y;
+    }else{
+        region_rootpos(reg, &pgeom->x, &pgeom->y);
+        pgeom->x=rgeom->x-pgeom->x;
+        pgeom->y=rgeom->y-pgeom->y;
+    }
+}
+
 /*}}}*/
 
