@@ -1,7 +1,7 @@
 /*
  * ion/ioncore/tags.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2005. 
+ * Copyright (c) Tuomo Valkonen 1999-2006. 
  *
  * Ion is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by
@@ -34,7 +34,7 @@ bool region_set_tagged(WRegion *reg, int sp)
             reg->flags|=REGION_TAGGED;
             objlist_insert_last(&taglist, (Obj*)reg);
         }
-        region_notify_change(reg);
+        region_notify_change(reg, "tag");
     }
 
     return nset;
@@ -80,6 +80,11 @@ void ioncore_clear_tags()
 /*{{{ Iteration */
 
 
+/*EXTL_DOC
+ * Returns first tagged object.
+ */
+EXTL_SAFE
+EXTL_EXPORT
 WRegion *ioncore_tags_first()
 {
     return (WRegion*)OBJLIST_FIRST(WRegion*, taglist);
@@ -92,12 +97,38 @@ WRegion *ioncore_tags_take_first()
     
     if(reg!=NULL){
         reg->flags&=~REGION_TAGGED;
-        region_notify_change(reg);
+        region_notify_change(reg, "tag");
     }
     
     return reg;
 }
 
+/*EXTL_DOC
+ * Returns a list of tagged regions.
+ */
+EXTL_SAFE
+EXTL_EXPORT
+ExtlTab ioncore_tagged_list()
+{
+    int n=0;
+    ExtlTab tab;
+    WRegion *region;
+    ObjListIterTmp tmp;
+
+    region=ioncore_tags_first();
+    if(!region)
+        return extl_table_none();
+
+    tab=extl_create_table();
+
+    FOR_ALL_ON_OBJLIST(WRegion*, region, taglist, tmp){
+        if(extl_table_seti_o(tab, n+1, (Obj*)region))
+            n++;
+    }
+
+    return tab;
+}
+
 
 /*}}}*/
 
