@@ -1,7 +1,7 @@
 /*
  * ion/ioncore/tags.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2004. 
+ * Copyright (c) Tuomo Valkonen 1999-2006. 
  *
  * Ion is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by
@@ -12,15 +12,14 @@
 #ifndef ION_IONCORE_TAGS_H
 #define ION_IONCORE_TAGS_H
 
+#include <libtu/setparam.h>
 #include "region.h"
 
-extern void region_tag(WRegion *reg);
-extern void region_untag(WRegion *reg);
-extern void region_toggle_tag(WRegion *reg);
-extern void clear_tags();
+extern bool region_set_tagged(WRegion *reg, int sp);
+extern bool region_is_tagged(WRegion *reg);
 
-extern WRegion *tag_first();
-extern WRegion *tag_take_first();
-extern WRegion *tag_next(WRegion *reg);
+extern void ioncore_clear_tags();
+extern WRegion *ioncore_tags_first();
+extern WRegion *ioncore_tags_take_first();
 
 #endif /* ION_IONCORE_TAGS_H */
