@@ -657,7 +657,6 @@ static void dockapp_calc_preferred_size(WDock *dock, int grow,
                                         const WRectangle *tile_size,
                                         WDockApp *da)
 {
-    XSizeHints hints;
     int w=da->geom.w, h=da->geom.h;
     
     if(grow==DOCK_GROW_UP || grow==DOCK_GROW_DOWN){
@@ -668,8 +667,7 @@ static void dockapp_calc_preferred_size(WDock *dock, int grow,
         da->geom.h=minof(h, tile_size->h);
     }
     
-    region_size_hints(da->reg, &hints);
-    xsizehints_correct(&hints, &(da->geom.w), &(da->geom.h), TRUE);
+    region_size_hints_correct(da->reg, &(da->geom.w), &(da->geom.h), TRUE);
 }
 
 
@@ -838,18 +836,13 @@ static void dock_managed_rqgeom(WDock *dock, WRegion *reg, int flags,
 }
 
 
-static void dock_rqgeom_clientwin(WDock *dock, WClientWin *cwin,
-                                  int rqflags, const WRectangle *geom)
+void dock_size_hints(WDock *dock, WSizeHints *hints)
 {
-    dock_managed_rqgeom_(dock, (WRegion*)cwin, rqflags, geom, NULL, FALSE);
-}
-
-
-void dock_size_hints(WDock *dock, XSizeHints *hints)
-{
-    hints->flags=PMinSize|PMaxSize;
+    hints->min_set=TRUE;
     hints->min_width=dock->min_w;
     hints->min_height=dock->min_h;
+    
+    hints->max_set=TRUE;
     hints->max_width=dock->max_w;
     hints->max_height=dock->max_h;
 }
@@ -857,18 +850,6 @@ void dock_size_hints(WDock *dock, XSizeHints *hints)
 
 static bool dock_fitrep(WDock *dock, WWindow *parent, const WFitParams *fp)
 {
-    WFitParams fp2;
-    
-    if(fp->mode&REGION_FIT_BOUNDS){
-        int pos, grow;
-        dock_get_pos_grow(dock, &pos, &grow);
-        fp2.mode=REGION_FIT_EXACT;
-        fp2.g.w=minof(dock->min_w, fp->g.w);
-        fp2.g.h=minof(dock->min_h, fp->g.h);
-        calc_dock_pos(&(fp2.g), &(fp->g), pos);
-        fp=&fp2;
-    }
-
     if(!window_fitrep(&(dock->win), parent, fp))
         return FALSE;
 
@@ -1118,6 +1099,8 @@ ExtlTab dock_get(WDock *dock)
 
 static bool dock_init(WDock *dock, WWindow *parent, const WFitParams *fp)
 {
+    WFitParams fp2=*fp;
+    
     dock->pos=dock_param_pos.dflt;
     dock->grow=dock_param_grow.dflt;
     dock->is_auto=dock_param_is_auto.dflt;
@@ -1130,7 +1113,8 @@ static bool dock_init(WDock *dock, WWindow *parent, const WFitParams *fp)
     dock->arrange_called=FALSE;
     dock->save=TRUE;
 
-    if(!window_init((WWindow*)dock, parent, fp))
+    
+    if(!window_init((WWindow*)dock, parent, &fp2))
         return FALSE;
     
     region_add_bindmap((WRegion*)dock, dock_bindmap);
@@ -1143,19 +1127,7 @@ static bool dock_init(WDock *dock, WWindow *parent, const WFitParams *fp)
 
     LINK_ITEM(docks, dock, dock_next, dock_prev);
     
-    /* Just calculate real min/max size */
-    dock_managed_rqgeom_(dock, NULL, 0, NULL, NULL, TRUE);
-    
-    if(fp->mode&REGION_FIT_BOUNDS){
-        WRectangle dg;
-        dg.w=minof(dock->min_w, fp->g.w);
-        dg.h=minof(dock->min_h, fp->g.h);
-        calc_dock_pos(&dg, &(fp->g), dock->pos);
-        window_do_fitrep((WWindow*)dock, NULL, &dg);
-    }
-
     return TRUE;
-
 }
 
 
@@ -1198,7 +1170,6 @@ WDock *mod_dock_create(ExtlTab tab)
     int screenid=0;
     WScreen *screen=NULL;
     WDock *dock=NULL;
-    WFitParams fp;
     WRegion *stdisp=NULL;
     WMPlexSTDispInfo din;
     
@@ -1237,35 +1208,69 @@ WDock *mod_dock_create(ExtlTab tab)
         }
     }
 
-    fp.g.x=0;
-    fp.g.y=0;
-    fp.g.w=1;
-    fp.g.h=1;
-    fp.mode=REGION_FIT_EXACT;
+    /* Create the dock */
     
-    dock=create_dock((WWindow*)screen, &fp);
+    if(floating){
+        WMPlexAttachParams par;
+        
+        par.flags=(MPLEX_ATTACH_UNNUMBERED
+                   |MPLEX_ATTACH_SIZEPOLICY
+                   |MPLEX_ATTACH_GEOM);
+        
+        par.szplcy=SIZEPOLICY_FREE;
+        par.geom.x=0;
+        par.geom.y=0;
+        par.geom.w=1;
+        par.geom.h=1;
+        
+        if(extl_table_is_bool_set(tab, "floating_hidden"))
+            par.flags|=MPLEX_ATTACH_HIDDEN;
+        
+        dock=(WDock*)mplex_do_attach_new((WMPlex*)screen, &par,
+                                         (WRegionCreateFn*)create_dock, 
+                                         NULL);
+    }else{
+        WFitParams fp;
+        
+        fp.mode=REGION_FIT_BOUNDS|REGION_FIT_WHATEVER;
+        fp.g.x=0;
+        fp.g.y=0;
+        fp.g.w=1;
+        fp.g.h=1;
+        
+        dock=create_dock((WWindow*)screen, &fp);
+    }
+
     if(dock==NULL){
         warn("Failed to create dock.");
         return NULL;
     }
     
+    /* Get parameters */
     dock->save=FALSE;
     dock_do_set(dock, tab, FALSE);
     
+    /* Final setup */    
     if(floating){
-        int af=MPLEX_ATTACH_L2|MPLEX_ATTACH_L2_PASSIVE;
-        if(extl_table_is_bool_set(tab, "floating_hidden"))
-            af|=MPLEX_ATTACH_L2_HIDDEN;
+        WRectangle dg;
+        const WRectangle *pg=&REGION_GEOM(screen);
         
-        if(mplex_attach_simple((WMPlex*)screen, (WRegion*)dock, af)!=NULL)
-            return dock;
+        /* Just calculate real min/max size */
+        dock_managed_rqgeom_(dock, NULL, 0, NULL, NULL, TRUE);
+        
+        dg.w=minof(dock->min_w, pg->w);
+        dg.h=minof(dock->min_h, pg->h);
+        calc_dock_pos(&dg, pg, dock->pos);
+        region_rqgeom((WRegion*)dock, 0, &dg, NULL);
+        
+        return dock;
     }else{
         mplexpos(dock->pos, &din.pos);
         din.fullsize=FALSE; /* not supported */
         if(mplex_set_stdisp((WMPlex*)screen, (WRegion*)dock, &din))
             return dock;
     }
-    
+
     /* Failed to attach. */
     warn("Failed to attach dock to screen.");
     destroy_obj((Obj*)dock);
@@ -1289,10 +1294,8 @@ void mod_dock_set_floating_shown_on(WMPlex *mplex, const char *how)
     WDock *dock;
     
     for(dock=docks; dock; dock=dock->dock_next){
-        if(REGION_MANAGER(dock)==(WRegion*)mplex &&
-           mplex_layer(mplex, (WRegion*)dock)==2){
-            mplex_l2_set_hidden(mplex, (WRegion*)dock, setpar);
-        }
+        if(REGION_MANAGER(dock)==(WRegion*)mplex)
+            mplex_set_hidden(mplex, (WRegion*)dock, setpar);
     }
 }
 
@@ -1335,13 +1338,11 @@ WRegion *dock_load(WWindow *par, const WFitParams *fp, ExtlTab tab)
 /*{{{ Client window management setup */
 
 
-static WDockApp *dock_do_attach_(WDock *dock, 
-                                 WRegionAttachHandler *handler,
-                                 void *handlerparams)
+static bool dock_do_attach_final(WDock *dock, WRegion *reg, void *unused)
 {
     WDockApp *dockapp, *before_dockapp;
+    WRectangle geom;
     WFitParams fp;
-    WRegion *reg;
     bool draw_border=TRUE;
     int pos=INT_MAX;
      
@@ -1349,20 +1350,8 @@ static WDockApp *dock_do_attach_(WDock *dock,
     dockapp=ALLOC(WDockApp);
     
     if(dockapp==NULL)
-        return NULL;
+        return FALSE;
     
-    dock_get_tile_size(dock, &(fp.g));
-    fp.g.x=0;
-    fp.g.y=0;
-    fp.mode=REGION_FIT_WHATEVER|REGION_FIT_BOUNDS;
-
-    reg=handler((WWindow*)dock, &fp, handlerparams);
-    
-    if(reg==NULL){
-        free(dockapp);
-        return NULL;
-    }
-
     if(OBJ_IS(reg, WClientWin)){
         ExtlTab proptab=((WClientWin*)reg)->proptab;
         extl_table_gets_b(proptab, CLIENTWIN_WINPROP_BORDER, &draw_border);
@@ -1389,32 +1378,30 @@ static WDockApp *dock_do_attach_(WDock *dock,
 
     region_set_manager(reg, (WRegion*)dock);
     
-    fp.g=REGION_GEOM(reg);
+    geom=REGION_GEOM(reg);
     dock_managed_rqgeom(dock, reg, 
                         REGION_RQGEOM_WEAK_X|REGION_RQGEOM_WEAK_Y,
-                        &(fp.g), NULL);
+                        &geom, NULL);
     
     region_map(reg);
 
-    return dockapp;
+    return TRUE;
 }
 
 
-static WRegion *dock_do_attach_simple(WDock *dock,
-                                      WRegionAttachHandler *handler,
-                                      void *handlerparams)
-{
-    WDockApp *da=dock_do_attach_(dock, handler, handlerparams);
-    return (da!=NULL ? da->reg : NULL);
-}
 
-
-static WRegion *dock_do_attach(WDock *dock,
-                               WRegionAttachHandler *handler,
-                               void *handlerparams,
-                               void *param UNUSED)
+static bool dock_do_attach(WDock *dock, WRegionAttachData *data)
 {
-    return dock_do_attach_simple(dock, handler, handlerparams);
+    WFitParams fp;
+    dock_get_tile_size(dock, &(fp.g));
+    fp.g.x=0;
+    fp.g.y=0;
+    fp.mode=REGION_FIT_WHATEVER|REGION_FIT_BOUNDS;
+    
+    return (region_attach_helper((WRegion*)dock, (WWindow*)dock, &fp,
+                                 (WRegionDoAttachFn*)dock_do_attach_final, 
+                                 NULL, data)
+            !=NULL);
 }
 
 
@@ -1424,10 +1411,13 @@ static WRegion *dock_do_attach(WDock *dock,
 EXTL_EXPORT_MEMBER
 bool dock_attach(WDock *dock, WRegion *reg)
 {
-    return (region__attach_reparent((WRegion*)dock, reg,
-                                    (WRegionDoAttachFn*)dock_do_attach, 
-                                    NULL)
-            !=NULL);
+    WRegionAttachData data;
+    WFitParams fp;
+    
+    data.type=REGION_ATTACH_REPARENT;
+    data.u.reg=reg;
+
+    return dock_do_attach(dock, &data);
 }
 
 
@@ -1438,11 +1428,17 @@ static bool dock_handle_drop(WDock *dock, int x, int y,
 }
 
 
-static WBasicPHolder *dock_managed_get_pholder(WDock *dock, WRegion *mgd)
+static bool dock_ph_handler(WDock *dock, int flags, WRegionAttachData *data)
 {
-    return create_basicpholder((WRegion*)dock, 
-                               ((WRegionDoAttachFnSimple*)
-                                dock_do_attach_simple));
+    return dock_do_attach(dock, data);
+}
+
+    
+static WPHolder *dock_managed_get_pholder(WDock *dock, WRegion *mgd)
+{
+    return (WPHolder*)create_basicpholder((WRegion*)dock,
+                                          ((WBasicPHolderHandler*)
+                                           dock_ph_handler));
 }
 
 
@@ -1452,10 +1448,10 @@ static WPHolder *dock_prepare_manage(WDock *dock, const WClientWin *cwin,
 {
     if(redir==MANAGE_REDIR_STRICT_YES)
         return NULL;
-    
-    return (WPHolder*)create_basicpholder((WRegion*)dock, 
-                                          ((WRegionDoAttachFnSimple*)
-                                           dock_do_attach_simple));
+
+    return (WPHolder*)create_basicpholder((WRegion*)dock,
+                                          ((WBasicPHolderHandler*)
+                                           dock_ph_handler));
 }
 
 
@@ -1463,17 +1459,16 @@ static void dock_managed_remove(WDock *dock, WRegion *reg)
 {
 
     WDockApp *dockapp=dock_find_dockapp(dock, reg);
-    if(dockapp!=NULL){
-        UNLINK_ITEM(dock->dockapps, dockapp, next, prev);
-        free(dockapp);
-    }else{
-        warn("Dockapp not found.");
-    }
+    
+    if(dockapp==NULL)
+        return;
+    
+    UNLINK_ITEM(dock->dockapps, dockapp, next, prev);
+    free(dockapp);
 
     region_unset_manager(reg, (WRegion*)dock);
 
     dock_resize(dock);
-
 }
 
 
@@ -1685,7 +1680,6 @@ static DynFunTab dock_dynfuntab[]={
     {(DynFun*)region_orientation, (DynFun*)dock_orientation},
     {(DynFun*)region_may_destroy, (DynFun*)dock_may_destroy},
     {(DynFun*)region_handle_drop, (DynFun*)dock_handle_drop},
-    {(DynFun*)region_rqgeom_clientwin, (DynFun*)dock_rqgeom_clientwin},
 
     {(DynFun*)region_managed_get_pholder,
      (DynFun*)dock_managed_get_pholder},
