@@ -1,7 +1,7 @@
 /*
  * ion/panews/panews.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2005. 
+ * Copyright (c) Tuomo Valkonen 1999-2006. 
  *
  * Ion is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by
@@ -15,16 +15,15 @@
 #include <ioncore/common.h>
 #include <ioncore/region.h>
 #include <ioncore/screen.h>
-#include <ioncore/genws.h>
 #include <libextl/extl.h>
 #include <ioncore/rectangle.h>
 
-#include <mod_ionws/ionws.h>
+#include <mod_tiling/tiling.h>
 
 
 INTRCLASS(WPaneWS);
 DECLCLASS(WPaneWS){
-    WIonWS ionws;
+    WTiling tiling;
 };
 
 
