@@ -55,14 +55,13 @@ WMessage *mod_query_message(WMPlex *mplex, const char *p)
         return NULL;
     
     par.flags=(MPLEX_ATTACH_SWITCHTO|
-               MPLEX_ATTACH_L2|
-               MPLEX_ATTACH_L2_SEMIMODAL|
+               MPLEX_ATTACH_MODAL|
+               MPLEX_ATTACH_UNNUMBERED|
                MPLEX_ATTACH_SIZEPOLICY);
     par.szplcy=SIZEPOLICY_FULL_BOUNDS;
 
-    return (WMessage*)mplex_do_attach(mplex, 
-                                      (WRegionAttachHandler*)create_wmsg,
-                                      (void*)p,
-                                      &par);
+    return (WMessage*)mplex_do_attach_new(mplex, &par,
+                                          (WRegionCreateFn*)create_wmsg,
+                                          (void*)p);
 }
 
