@@ -1,7 +1,7 @@
 --
 -- ion/query/mod_query.lua -- Some common queries for Ion
 -- 
--- Copyright (c) Tuomo Valkonen 2004-2005.
+-- Copyright (c) Tuomo Valkonen 2004-2006.
 -- 
 -- Ion is free software; you can redistribute it and/or modify it under
 -- the terms of the GNU Lesser General Public License as published by
@@ -10,10 +10,10 @@
 --
 
 
--- This is a slight abuse of the _LOADED variable perhaps, but library-like 
--- packages should handle checking if they're loaded instead of confusing 
--- the user with require/include differences.
-if _LOADED["mod_query"] then return end
+-- This is a slight abuse of the package.loaded variable perhaps, but
+-- library-like packages should handle checking if they're loaded instead of
+-- confusing the user with require/include differences.
+if package.loaded["mod_query"] then return end
 
 if not ioncore.load_module("mod_query") then
     return
@@ -24,7 +24,8 @@ local mod_query=_G["mod_query"]
 assert(mod_query)
 
 
-local DIE_TIMEOUT=30*1000 -- 30 seconds
+local DIE_TIMEOUT_ERRORCODE=10 -- 10 seconds
+local DIE_TIMEOUT_NO_ERRORCODE=2 -- 2 seconds
 
 
 -- Generic helper functions {{{
@@ -37,23 +38,41 @@ function mod_query.make_completor(completefn)
     return completor
 end
 
-    
+
+--DOC
+-- Low-level query routine. \var{mplex} is the \type{WMPlex} to display
+-- the query in, \var{prompt} the prompt string, and \var{initvalue}
+-- the initial contents of the query box. \var{handler} is a function
+-- that receives (\var{mplex}, result string) as parameter when the
+-- query has been succesfully completed, \var{completor} the completor
+-- routine which receives a (\var{cp}, \var{str}, \var{point}) as parameters.
+-- The parameter \var{str} is the string to be completed and \var{point}
+-- cursor's location within it. Completions should be eventually,
+-- possibly asynchronously, set with \fnref{WComplProxy.set_completions} 
+-- on \var{cp}.
 function mod_query.query(mplex, prompt, initvalue, handler, completor,
                          context)
     local function handle_it(str)
         handler(mplex, str)
     end
+    local function cycle(wedln)
+        wedln:complete('next', 'normal')
+    end
+
     -- Check that no other queries are open in the mplex.
-    local l=mplex:llist(2)
-    for i, r in l do
+    local l=mplex:managed_list()
+    for i, r in pairs(l) do
         if obj_is(r, "WEdln") then
             return
         end
     end
-    wedln=mod_query.do_query(mplex, prompt, initvalue, handle_it, completor)
+    local wedln=mod_query.do_query(mplex, prompt, initvalue, 
+                                   handle_it, completor, cycle)
     if context then
         wedln:set_context(context)
     end
+    
+    return wedln
 end
 
 
@@ -74,19 +93,42 @@ end
 
 local errdata={}
 
---[[
-local function chld_handler(pid)
+local function maybe_finish(pid)
     local t=errdata[pid]
-    if t then
+
+    if t and t.closed and t.dietime then
         errdata[pid]=nil
-        if t.errs then
+        local tmd=os.difftime(t.dietime, t.starttime)
+        --if tmd<DIE_TIMEOUT_ERRORCODE and t.signaled then
+        --    local msg=TR("Program received signal ")..t.termsig.."\n"
+        --    mod_query.warn(t.mplex, msg..(t.errs or ""))
+        --else
+        if ((tmd<DIE_TIMEOUT_ERRORCODE and (t.hadcode or t.signaled)) or
+                (tmd<DIE_TIMEOUT_NO_ERRORCODE)) and t.errs then
             mod_query.warn(t.mplex, t.errs)
         end
     end
 end
 
+
+local badsig_={4, 5, 6, 7, 8, 11}
+local badsig={}
+for _, v in pairs(badsig_) do 
+    badsig[v]=true 
+end
+
+local function chld_handler(p)
+    local t=errdata[p.pid]
+    if t then
+        t.dietime=os.time()
+        t.signaled=(p.signaled and badsig[p.termsig])
+        t.termsig=p.termsig
+        t.hadcode=(p.exited and p.exitstatus~=0)
+        maybe_finish(pid)
+    end
+end
+
 ioncore.get_hook("ioncore_sigchld_hook"):add(chld_handler)
-]]
 
 function mod_query.exec_on_merr(mplex, cmd)
     local pid
@@ -97,9 +139,9 @@ function mod_query.exec_on_merr(mplex, cmd)
             if t then
                 if str then
                     t.errs=(t.errs or "")..str
-                elseif t.errs then
-                    errdata[pid]=nil
-                    mod_query.warn(t.mplex, t.errs)
+                else
+                    t.closed=true
+                    maybe_finish(pid)
                 end
             end
         end
@@ -116,9 +158,11 @@ function mod_query.exec_on_merr(mplex, cmd)
     end
 
     local tmr=ioncore.create_timer();
-    tmr:set(DIE_TIMEOUT, timeout)
+    local tmd=math.max(DIE_TIMEOUT_NO_ERRORCODE, DIE_TIMEOUT_ERRORCODE)
+    local now=os.time()  
+    tmr:set(tmd*1000, timeout)
     
-    errdata[pid]={tmr=tmr, mplex=mplex}
+    errdata[pid]={tmr=tmr, mplex=mplex, starttime=now}
 end
 
 
@@ -126,8 +170,7 @@ function mod_query.file_completor(wedln, str)
     local ic=ioncore.lookup_script("ion-completefile")
     if ic then
         mod_query.popen_completions(wedln,
-                                   ic..(wp or " ")..string.shell_safe(str),
-                                   "")
+                                   ic.." "..string.shell_safe(str))
     end
 end
 
@@ -174,36 +217,13 @@ end
 local MAXDEPTH=10
 
 
-function mod_query.lookup_workspace_classes()
-    local classes={}
-    
-    for k, v in _G do
-        if type(v)=="table" and v.__typename then
-            v2=v.__parentclass
-            for i=1, MAXDEPTH do
-                if not v2 then 
-                    break
-                end
-                if v2.__typename=="WGenWS" then
-                    table.insert(classes, v.__typename)
-                    break
-                end
-                v2=v2.__parentclass
-            end
-        end
-    end
-    
-    return classes
-end
-
-
 function mod_query.complete_from_list(list, str)
     local results={}
     local len=string.len(str)
     if len==0 then
         results=list
     else
-        for _, m in list do
+        for _, m in pairs(list) do
             if string.sub(m, 1, len)==str then
                 table.insert(results, m)
             end
@@ -220,14 +240,27 @@ mod_query.COLLECT_THRESHOLD=2000
 
 --DOC
 -- This function can be used to read completions from an external source.
--- The string \var{cmd} is a shell command to be executed. To its  stdout, 
+-- The parameter \var{cp} is the completion proxy to be used,
+-- and the string \var{cmd} the shell command to be executed. To its stdout, 
 -- the command should on the first line write the \var{common_beg} 
--- parameter of \fnref{WComplProxy.set_completions} and a single actual 
--- completion on each of the successive lines.
-function mod_query.popen_completions(cp, cmd, beg, fn)
+-- parameter of \fnref{WComplProxy.set_completions} (which \var{fn} maybe used
+-- to override) and a single actual completion on each of the successive lines.
+-- The function \var{reshnd} may be used to override a result table
+-- building routine.
+function mod_query.popen_completions(cp, cmd, fn, reshnd)
     
     local pst={cp=cp, maybe_stalled=0}
     
+    if not reshnd then
+        reshnd = function(rs, a)
+                     if not rs.common_beg then
+                         rs.common_beg=a
+                     else
+                         table.insert(rs, a)
+                     end
+                 end
+    end
+
     local function rcv(str)
         local data=""
         local results={}
@@ -245,19 +278,13 @@ function mod_query.popen_completions(cp, cmd, beg, fn)
             if totallen>ioncore.RESULT_DATA_LIMIT then
                 error(TR("Too much result data"))
             end
+            
 
-            data=string.gsub(data..str, "([^\n]*)\n",
-                             function(a)
-                                 -- ion-completefile will return possible 
-                                 -- common part of path on  the first line 
-                                 -- and the entries in that directory on the
-                                 -- following lines.
-                                 if not results.common_beg then
-                                     results.common_beg=(beg or "")..a
-                                 else
-                                     table.insert(results, a)
-                                 end
+            data=string.gsub(data..str, "([^\n]*)\n", 
+                             function(s) 
+                                 reshnd(results, s) 
                                  lines=lines+1
+                                 return ""
                              end)
             
             if lines>mod_query.COLLECT_THRESHOLD then
@@ -282,7 +309,7 @@ function mod_query.popen_completions(cp, cmd, beg, fn)
     
     local found_clean=false
     
-    for k, v in pipes do
+    for k, v in pairs(pipes) do
         if v.cp==cp then
             if v.maybe_stalled<2 then
                 v.maybe_stalled=v.maybe_stalled+1
@@ -298,23 +325,32 @@ function mod_query.popen_completions(cp, cmd, beg, fn)
 end
 
 
---}}}
+-- }}}
 
 
 -- Simple queries for internal actions {{{
 
 
+function mod_query.call_warn(mplex, fn)
+    local err = collect_errors(fn)
+    if err then
+        mod_query.warn(mplex, err)
+    end
+    return err
+end
+
+
 function mod_query.complete_name(str, list)
     local entries={}
     local l=string.len(str)
-    for i, reg in list do
+    for i, reg in pairs(list) do
         local nm=reg:name()
         if nm and string.sub(nm, 1, l)==str then
             table.insert(entries, nm)
         end
     end
-    if table.getn(entries)==0 then
-        for i, reg in list do
+    if #entries==0 then
+        for i, reg in pairs(list) do
             local nm=reg:name()
             if nm and string.find(nm, str, 1, true) then
                 table.insert(entries, nm)
@@ -329,7 +365,7 @@ function mod_query.complete_clientwin(str)
 end
 
 function mod_query.complete_workspace(str)
-    return mod_query.complete_name(str, ioncore.region_list("WGenWS"))
+    return mod_query.complete_name(str, ioncore.region_list("WGroupWS"))
 end
 
 function mod_query.complete_region(str)
@@ -354,54 +390,33 @@ function mod_query.attachclient_handler(frame, str)
         mod_query.warn(frame, TR("Could not find client window %s.", str))
     elseif frame:rootwin_of()~=cwin:rootwin_of() then
         mod_query.warn(frame, TR("Cannot attach: different root windows."))
+    elseif cwin:manager()==frame then
+        cwin:goto()
     else
-        frame:attach(cwin, { switchto = true })
+        mod_query.call_warn(frame, 
+                            function() 
+                                frame:attach(cwin, { switchto = true }) 
+                            end)
     end
 end
 
 
 function mod_query.workspace_handler(mplex, name)
-    local ws=ioncore.lookup_region(name, "WGenWS")
+    local ws=ioncore.lookup_region(name, "WGroupWS")
     if ws then
         ws:goto()
         return
     end
+
+    local scr=mplex:screen_of()
     
-    local classes=mod_query.lookup_workspace_classes()
-    
-    local function completor(cp, what)
-        local results=mod_query.complete_from_list(classes, what)
-        cp:set_completions(results)
-    end
-    
-    local function handler(mplex, cls)
-        local scr=mplex:screen_of()
-        if not scr then
-            mod_query.warn(mplex, TR("Unable to create workspace: no screen."))
-            return
-        end
-        
-        if not cls or cls=="" then
-            cls=ioncore.get().default_ws_type
-        end
-        
-        local err=collect_errors(function()
-                                     ws=scr:attach_new({ 
-                                         type=cls, 
-                                         name=name, 
-                                         switchto=true 
-                                     })
-                                 end)
-        if not ws then
-            mod_query.warn(mplex, err or TR("Unknown error"))
+    local function mkws()
+	if not ioncore.create_ws(scr, {name=name}) then
+            error(TR("Unknown error"))
         end
     end
-    
-    local defcls=ioncore.get().default_ws_type
-    local prompt=TR("Workspace type (%s):", defcls or TR("none"))
-    
-    mod_query.query(mplex, prompt, "", handler, completor,
-                    "workspacename")
+
+    mod_query.call_warn(mplex, mkws)
 end
 
 
@@ -430,7 +445,7 @@ end
 
 --DOC
 -- This query asks for the name of a workspace. If a workspace
--- (an object inheriting \type{WGenWS}) with such a name exists,
+-- (an object inheriting \type{WGroupWS}) with such a name exists,
 -- it will be switched to. Otherwise a new workspace with the
 -- entered name will be created and the user will be queried for
 -- the type of the workspace.
@@ -474,14 +489,14 @@ end
 -- This function asks for a name new for the workspace on which the
 -- query resides.
 function mod_query.query_renameworkspace(mplex)
-    local ws=ioncore.find_manager(mplex, "WGenWS")
+    local ws=ioncore.find_manager(mplex, "WGroupWS")
     mod_query.query(mplex, TR("Workspace name:"), ws:name(),
                     function(mplex, str) ws:set_name(str) end,
                     nil, "framename")
 end
 
 
---}}}
+-- }}}
 
 
 -- Run/view/edit {{{
@@ -510,12 +525,17 @@ function mod_query.query_runfile(mplex, script, prompt)
 end
 
 
-function break_cmdline(str, no_ws)
-    local st, en, beg, rest, ch
+local function isspace(s)
+    return string.find(s, "^%s*$")~=nil
+end
+
+
+local function break_cmdline(str, no_ws)
+    local st, en, beg, rest, ch, rem
     local res={""}
 
     local function ins(str)
-        local n=table.getn(res)
+        local n=#res
         if string.find(res[n], "^%s+$") then
             table.insert(res, str)
         else
@@ -524,13 +544,13 @@ function break_cmdline(str, no_ws)
     end
 
     local function ins_space(str)
-        local n=table.getn(res)
+        local n=#res
         if no_ws then
             if res[n]~="" then
                 table.insert(res, "")
             end
         else
-            if string.find(res[n], "^%s*$") then
+            if isspace(res[n]) then
                 res[n]=res[n]..str
             else
                 table.insert(res, str)
@@ -538,6 +558,7 @@ function break_cmdline(str, no_ws)
         end
     end
 
+    -- Handle terminal startup syntax
     st, en, beg, ch, rest=string.find(str, "^(%s*)(:+)(.*)")
     if beg then
         if string.len(beg)>0 then
@@ -547,14 +568,14 @@ function break_cmdline(str, no_ws)
         ins_space("")
         str=rest
     end
-            
+
     while str~="" do
-        st, en, beg, rest, ch=string.find(str, "^(.-)(([%s'\"\\]).*)")
+        st, en, beg, rest, ch=string.find(str, "^(.-)(([%s'\"\\|])(.*))")
         if not beg then
             ins(str)
             break
         end
-        
+
         ins(beg)
         str=rest
         
@@ -571,7 +592,13 @@ function break_cmdline(str, no_ws)
         elseif ch=="'" then
             st, en, beg, rest=string.find(str, "^('.-')(.*)")
         else
-            st, en, beg, rest=string.find(str, "^(%s*)(.*)")
+            if ch=='|' then
+                ins_space('')
+                ins(ch)
+            else -- ch==' '
+                ins_space(ch)
+            end
+            st, en, beg, rest=string.find(str, "^.(%s*)(.*)")
             assert(beg and rest)
             ins_space(beg)
             sp=true
@@ -601,7 +628,7 @@ end
 
 
 local function quote(str)
-    return string.gsub(str, "([%(%)\"'\\%*%?%[%] ])", "\\%1")
+    return string.gsub(str, "([%(%)\"'\\%*%?%[%]%| ])", "\\%1")
 end
 
 
@@ -612,7 +639,7 @@ local function find_point(strs, point)
             return i
         end
     end
-    return table.getn(strs)
+    return #strs
 end
 
 
@@ -634,12 +661,12 @@ function mod_query.exec_completor(wedln, str, point)
         s_beg=s_beg..parts[i]
     end
     
-    for i=complidx+1, table.getn(parts) do
+    for i=complidx+1, #parts do
         s_end=s_end..parts[i]
     end
     
     local wp=" "
-    if complidx==1 then
+    if complidx==1 or (complidx==2 and isspace(parts[1])) then
         wp=" -wp "
     elseif string.find(parts[1], "^:+$") then
         if complidx==2 then
@@ -658,11 +685,23 @@ function mod_query.exec_completor(wedln, str, point)
         cp:set_completions(res)
     end
 
+    local function filter_fn(res, s)
+        if not res.common_beg then
+            if s=="./" then
+                res.common_beg=""
+            else
+                res.common_beg=s
+            end
+        else
+            table.insert(res, s)
+        end
+    end
+    
     local ic=ioncore.lookup_script("ion-completefile")
     if ic then
         mod_query.popen_completions(wedln,
                                    ic..wp..string.shell_safe(s_compl),
-                                   "", set_fn)
+                                   set_fn, filter_fn)
     end
 end
 
@@ -842,8 +881,8 @@ end
 function mod_query.man_completor(wedln, str)
     local mc=ioncore.lookup_script("ion-completeman")
     if mc then
-        mod_query.popen_completions(wedln, mc.." -complete "..
-                                    string.shell_safe(str))
+        mod_query.popen_completions(wedln, (mc.." -complete "
+                                            ..string.shell_safe(str)))
     end
 end
 
@@ -873,21 +912,37 @@ function mod_query.create_run_env(mplex)
     local env={
         _=mplex, 
         _sub=mplex:current(),
+        print=my_print
     }
     setmetatable(env, meta)
     return env
 end
 
 function mod_query.do_handle_lua(mplex, env, code)
+    local print_res
+    local function collect_print(...)
+        local tmp=""
+        local l=#arg
+        for i=1,l do
+            tmp=tmp..tostring(arg[i])..(i==l and "\n" or "\t")
+        end
+        print_res=(print_res and print_res..tmp or tmp)
+    end
+
     local f, err=loadstring(code)
     if not f then
         mod_query.warn(mplex, err)
         return
     end
+    
+    env.print=collect_print
     setfenv(f, env)
+    
     err=collect_errors(f)
     if err then
         mod_query.warn(mplex, err)
+    elseif print_res then
+        mod_query.message(mplex, print_res)
     end
 end
 
@@ -937,14 +992,16 @@ function mod_query.do_complete_lua(env, str)
     local tab=comptab
     local seen={}
     while true do
-        for k in tab do
-            if type(k)=="string" then
-                if string.sub(k, 1, l)==tocomp then
-                    table.insert(compl, k)
+        if type(tab) == "table" then
+            for k in pairs(tab) do
+                if type(k)=="string" then
+                    if string.sub(k, 1, l)==tocomp then
+                        table.insert(compl, k)
+                    end
                 end
             end
         end
-        
+
         -- We only want to display full list of functions for objects, not 
         -- the tables representing the classes.
         --if not metas then break end
@@ -956,7 +1013,7 @@ function mod_query.do_complete_lua(env, str)
     
     -- If there was only one completion and it is a string or function,
     -- concatenate it with "." or "(", respectively.
-    if table.getn(compl)==1 then
+    if #compl==1 then
         if type(comptab[compl[1]])=="table" then
             compl[1]=compl[1] .. "."
         elseif type(comptab[compl[1]])=="function" then
@@ -984,7 +1041,7 @@ function mod_query.query_lua(mplex)
         return mod_query.do_handle_lua(mplex, env, code)
     end
     
-    mod_query.query(mplex, TR("Lua code: "), nil, handler, complete, "lua")
+    mod_query.query(mplex, TR("Lua code:"), nil, handler, complete, "lua")
 end
 
 -- }}}
@@ -994,35 +1051,62 @@ end
 
 --DOC
 -- This query can be used to create a query of a defined menu.
-function mod_query.query_menu(mplex, prompt, menuname)
+function mod_query.query_menu(mplex, themenu, prompt)
     local _sub=mplex:current()
-    local menu=mod_menu.evalmenu(menuname, {mplex, _sub})
+    local menu=ioncore.evalmenu(themenu, {mplex, _sub})
+    local menuname=(type(themenu)=="string" and themenu or "?")
     
     if not menu then
-        mod_query.warn(mplex, TR("Unknown menu %s.", tostring(menuname)))
+        mod_query.warn(mplex, TR("Unknown menu %s.", tostring(themenu)))
         return
     end
     
-    function complete(str)
+    if not prompt then
+        prompt=menuname..":"
+    end
+
+    local function xform_name(n, is_submenu)
+        return (string.lower(string.gsub(n, "[-%s]+", "-"))
+                ..(is_submenu and "/" or ""))
+    end
+
+    local function xform_menu(t, m, p)
+        for _, v in ipairs(m) do
+            if v.name then
+                local is_submenu=v.submenu_fn
+                local n=p..xform_name(v.name, is_submenu)
+                while t[n] do
+                    n=n.."'"
+                end
+                t[n]=v
+                if is_submenu and not v.noautoexpand then
+                    local sm=v.submenu_fn()
+                    if sm then
+                        xform_menu(t, sm, n)
+                    else
+                        ioncore.warn_traced(TR("Missing submenu ")
+                                            ..(v.name or ""))
+                    end
+                end
+            end
+        end
+        return t
+    end
+    
+    local ntab=xform_menu({}, menu, "")
+    
+    local function complete(str)
         local results={}
-        local len=string.len(str)
-        for _, m in menu do
-            local mn=m.name
-            if len==0 or string.sub(mn, 1, len)==str then
-                table.insert(results, mn)
+        for s, e in pairs(ntab) do
+            if string.find(s, str, 1, true) then
+                table.insert(results, s)
             end
         end
         return results
-    end    
+    end
     
     local function handle(mplex, str)
-        local e
-        for k, v in menu do
-            if v.name==str then
-                e=v
-                break
-            end
-        end
+        local e=ntab[str]
         if e then
             if e.func then
                 local err=collect_errors(function() 
@@ -1032,8 +1116,8 @@ function mod_query.query_menu(mplex, prompt, menuname)
                     mod_query.warn(mplex, err)
                 end
             elseif e.submenu_fn then
-                mod_query.query_menu(mplex, TR("%s menu:", e.name), 
-                                     e.submenu_fn())
+                mod_query.query_menu(mplex, e.submenu_fn(),
+                                     TR("%s:", e.name))
             end
         else
             mod_query.warn(mplex, TR("No entry '%s'", str))
@@ -1041,7 +1125,7 @@ function mod_query.query_menu(mplex, prompt, menuname)
     end
     
     mod_query.query(mplex, prompt, nil, handle, 
-                    mod_query.make_completor(complete), "menu")
+                    mod_query.make_completor(complete), "menu."..menuname)
 end
 
 -- }}}
@@ -1058,30 +1142,46 @@ end
 
 
 --DOC
--- Show information about a client window.
-function mod_query.show_clientwin(mplex, cwin)
+-- Show information about a region tree 
+function mod_query.show_tree(mplex, reg, max_depth)
     local function indent(s)
         local i="    "
         return i..string.gsub(s, "\n", "\n"..i)
     end
     
-    local function get_info(cwin)
+    local function get_info(reg, indent, d)
+        if not reg then
+            return (indent .. "No region")
+        end
+        
         local function n(s) return (s or "") end
-        local i=cwin:get_ident()
-        local s=TR("Title: %s\nClass: %s\nRole: %s\nInstance: %s\nXID: 0x%x",
-                   n(cwin:name()), n(i.class), n(i.role), n(i.instance), 
-                   cwin:xid())
-        local t=TR("\nTransients:\n")
-        for k, v in cwin:managed_list() do
-            if obj_is(v, "WClientWin") then
-                s=s..t..indent(get_info(v))
-                t="\n"
+
+        local s=string.format("%s%s \"%s\"", indent, obj_typename(reg),
+                              n(reg:name()))
+        indent = indent .. "  "
+        if obj_is(reg, "WClientWin") then
+            local i=reg:get_ident()
+            s=s .. TR("\n%sClass: %s\n%sRole: %s\n%sInstance: %s\n%sXID: 0x%x",
+                      indent, n(i.class), 
+                      indent, n(i.role), 
+                      indent, n(i.instance), 
+                      indent, reg:xid())
+        end
+        
+        if (not max_depth or max_depth > d) and reg.managed_list then
+            local mgd=reg:managed_list()
+            if #mgd > 0 then
+                s=s .. "\n" .. indent .. "---"
+                for k, v in pairs(mgd) do
+                    s=s .. "\n" .. get_info(v, indent, d+1)
+                end
             end
         end
+        
         return s
     end
     
-    mod_query.message(mplex, get_info(cwin))
+    mod_query.message(mplex, get_info(reg, "", 0))
 end
 
 -- }}}
@@ -1090,7 +1190,7 @@ end
 dopath('mod_query_chdir')
 
 -- Mark ourselves loaded.
-_LOADED["mod_query"]=true
+package.loaded["mod_query"]=true
 
 
 -- Load configuration file
