@@ -1,7 +1,7 @@
 --
 -- ion/query/mod_query_chdir.lua
 -- 
--- Copyright (c) Tuomo Valkonen 2004-2005.
+-- Copyright (c) Tuomo Valkonen 2004-2006.
 -- 
 -- Ion is free software; you can redistribute it and/or modify it under
 -- the terms of the GNU Lesser General Public License as published by
@@ -10,8 +10,11 @@
 --
 
 local function ws_chdir(mplex, params)
-    ws=assert(ioncore.find_manager(mplex, "WGenWS"))
-    ioncore.chdir_for(ws, params[1] or "")
+    ws=assert(ioncore.find_manager(mplex, "WGroupWS"))
+    local ok, err=ioncore.chdir_for(ws, params[1] or "")
+    if not ok then
+        mod_query.warn(mplex, err)
+    end
 end
 
 local function ws_showdir(mplex, params)
