@@ -9,26 +9,28 @@
  * (at your option) any later version.
  */
 
+#include <string.h>
+
 #include <libtu/map.h>
+#include <libtu/minmax.h>
 #include <libextl/readconfig.h>
 #include <libmainloop/hooks.h>
-#include <ioncore/conf-bindings.h>
+
 #include <ioncore/saveload.h>
 #include <ioncore/screen.h>
 #include <ioncore/mplex.h>
 #include <ioncore/ioncore.h>
 #include <ioncore/global.h>
 #include <ioncore/framep.h>
-#include <ioncore/bindmaps.h>
+#include <ioncore/frame.h>
+#include <ioncore/names.h>
 
 #include "main.h"
-#include "scratchpad.h"
 #include "exports.h"
 
 
 /*{{{ Module information */
 
-
 #include "../version.h"
 
 char mod_sp_ion_api_version[]=ION_API_VERSION;
@@ -37,19 +39,10 @@ char mod_sp_ion_api_version[]=ION_API_VERSION;
 /*}}}*/
 
 
-/*{{{ Bindmaps w/ config */
+/*{{{ Bindmaps, config, etc. */
 
 
-WBindmap *mod_sp_scratchpad_bindmap=NULL;
-
-
-static StringIntMap frame_areas[]={
-    {"border",     FRAME_AREA_BORDER},
-    {"tab",        FRAME_AREA_TAB},
-    {"empty_tab",  FRAME_AREA_TAB},
-    {"client",     FRAME_AREA_CLIENT},
-    END_STRINGINTMAP
-};
+#define SP_NAME  "*scratchpad*"
 
 
 /*}}}*/
@@ -58,23 +51,61 @@ static StringIntMap frame_areas[]={
 /*{{{ Exports */
 
 
-static WScratchpad *create(WScreen *scr, int flags)
+static WRegion *create_frame_scratchpad(WWindow *parent, const WFitParams *fp,
+                                        void *unused)
 {
-    WScratchpad *sp;
+    return (WRegion*)create_frame(parent, fp, FRAME_MODE_UNKNOWN);
+}
+
+
+static WFrame *create(WMPlex *mplex, int flags)
+{
+    WFrame *sp;
+    WMPlexAttachParams par;
+    int sw=REGION_GEOM(mplex).w, sh=REGION_GEOM(mplex).h;
+
+    par.flags=(flags
+               |MPLEX_ATTACH_UNNUMBERED
+               |MPLEX_ATTACH_MODAL
+               |MPLEX_ATTACH_SIZEPOLICY
+               |MPLEX_ATTACH_GEOM);
+    par.szplcy=SIZEPOLICY_FREE_GLUE;
     
-    sp=(WScratchpad*)mplex_attach_hnd((WMPlex*)scr,
-                                      (WRegionAttachHandler*)
-                                      create_scratchpad, NULL,
-                                      (MPLEX_ATTACH_L2|flags));
+    par.geom.w=minof(sw, CF_SCRATCHPAD_DEFAULT_W);
+    par.geom.h=minof(sh, CF_SCRATCHPAD_DEFAULT_H);
+    par.geom.x=(sw-par.geom.w)/2;
+    par.geom.y=(sh-par.geom.h)/2;
+
+    sp=(WFrame*)mplex_do_attach_new((WMPlex*)mplex, &par,
+                                    create_frame_scratchpad,
+                                    NULL);
+    
+
     if(sp==NULL){
-        warn(TR("Unable to create scratchpad for screen %d."),
-             screen_id(scr));
+        warn(TR("Unable to create scratchpad."));
     }
     
+    region_set_name((WRegion*)sp, SP_NAME);
+    
     return sp;
 }
 
 
+static bool is_scratchpad(WRegion *reg)
+{
+    char *nm=reg->ni.name;
+    int inst_off=reg->ni.inst_off;
+    
+    if(nm==NULL)
+        return FALSE;
+    
+    if(inst_off<0)
+        return (strcmp(nm, SP_NAME)==0);
+    else
+        return (strncmp(nm, SP_NAME, inst_off)==0);
+}
+
+
 /*EXTL_DOC
  * Change displayed status of some scratchpad on \var{mplex} if one is 
  * found. The parameter \var{how} is one of (set/unset/toggle).
@@ -82,27 +113,25 @@ static WScratchpad *create(WScreen *scr, int flags)
 EXTL_EXPORT
 bool mod_sp_set_shown_on(WMPlex *mplex, const char *how)
 {
-    int i;
     int setpar=libtu_setparam_invert(libtu_string_to_setparam(how));
-    WScratchpad *sp;
-    WScreen *scr;
+    WMPlexIterTmp tmp;
+    WRegion *reg;
+    bool found=FALSE;
     
-    for(i=mplex_lcount(mplex, 2)-1; i>=0; i--){
-        sp=OBJ_CAST(mplex_lnth(mplex, 2, i), WScratchpad);
-        if(sp!=NULL)
-            return mplex_l2_set_hidden(mplex, (WRegion*)sp, setpar);
+    FOR_ALL_MANAGED_BY_MPLEX(mplex, reg, tmp){
+        if(is_scratchpad(reg)){
+            mplex_set_hidden(mplex, reg, setpar);
+            found=TRUE;
+        }
     }
-   
-    /* No scratchpad found; create one if a screen */
-    
-    scr=OBJ_CAST(mplex, WScreen);
-    if(scr!=NULL){
-        sp=create(scr, 0);
-        if(sp!=NULL)
-            return TRUE;
+
+    if(!found){
+        int sp=libtu_string_to_setparam(how);
+        if(sp==SETPARAM_SET || sp==SETPARAM_TOGGLE)
+            found=(create(mplex, 0)!=NULL);
     }
     
-    return FALSE;
+    return found;
 }
 
 
@@ -111,13 +140,13 @@ bool mod_sp_set_shown_on(WMPlex *mplex, const char *how)
  * The parameter \var{how} is one of (set/unset/toggle).
  */
 EXTL_EXPORT
-bool mod_sp_set_shown(WScratchpad *sp, const char *how)
+bool mod_sp_set_shown(WFrame *sp, const char *how)
 {
     if(sp!=NULL){
         int setpar=libtu_setparam_invert(libtu_string_to_setparam(how));
         WMPlex *mplex=OBJ_CAST(REGION_MANAGER(sp), WMPlex);
         if(mplex!=NULL)
-            return mplex_l2_set_hidden(mplex, (WRegion*)sp, setpar);
+            return mplex_set_hidden(mplex, (WRegion*)sp, setpar);
     }
     
     return FALSE;
@@ -132,36 +161,26 @@ bool mod_sp_set_shown(WScratchpad *sp, const char *how)
 
 void mod_sp_deinit()
 {
-    if(mod_sp_scratchpad_bindmap!=NULL){
-        ioncore_free_bindmap("WScratchpad", mod_sp_scratchpad_bindmap);
-        mod_sp_scratchpad_bindmap=NULL;
-    }
     mod_sp_unregister_exports();
-    ioncore_unregister_regclass(&CLASSDESCR(WScratchpad));
 }
 
 
 static void check_and_create()
 {
+    WMPlexIterTmp tmp;
     WScreen *scr;
-    int i;
+    WRegion *reg;
     
     /* No longer needed, free the memory the list uses. */
     hook_remove(ioncore_post_layout_setup_hook, check_and_create);
     
     FOR_ALL_SCREENS(scr){
-        WScratchpad *sp=NULL;
-        /* This is really inefficient, but most likely there's just
-         * the scratchpad on the list... 
-         */
-        for(i=0; i<mplex_lcount((WMPlex*)scr, 2); i++){
-            sp=OBJ_CAST(mplex_lnth((WMPlex*)scr, 2, i), WScratchpad);
-            if(sp!=NULL)
-                break;
+        FOR_ALL_MANAGED_BY_MPLEX((WMPlex*)scr, reg, tmp){
+            if(is_scratchpad(reg))
+                return;
         }
         
-        if(sp==NULL)
-            sp=create(scr, MPLEX_ATTACH_L2_HIDDEN);
+        create(&scr->mplex, MPLEX_ATTACH_HIDDEN);
     }
 }
     
@@ -171,19 +190,6 @@ bool mod_sp_init()
     if(!mod_sp_register_exports())
         return FALSE;
 
-    mod_sp_scratchpad_bindmap=ioncore_alloc_bindmap("WScratchpad", NULL);
-    
-    if(mod_sp_scratchpad_bindmap==NULL){
-        mod_sp_deinit();
-        return FALSE;
-    }
-    
-    if(!ioncore_register_regclass(&CLASSDESCR(WScratchpad),
-                                  (WRegionLoadCreateFn*)scratchpad_load)){
-        mod_sp_deinit();
-        return FALSE;
-    }
-    
     extl_read_config("cfg_sp", NULL, FALSE);
     
     if(ioncore_g.opmode==IONCORE_OPMODE_INIT){
