@@ -54,8 +54,6 @@ bool statusbar_init(WStatusBar *p, WWindow *parent, const WFitParams *fp)
     if(!window_init(&(p->wwin), parent, fp))
         return FALSE;
 
-    region_register((WRegion*)p);
-
     p->brush=NULL;
     p->elems=NULL;
     p->nelems=0;
@@ -75,6 +73,8 @@ bool statusbar_init(WStatusBar *p, WWindow *parent, const WFitParams *fp)
     }
     
     window_select_input(&(p->wwin), IONCORE_EVENTMASK_NORMAL);
+    
+    region_register((WRegion*)p);
 
     region_add_bindmap((WRegion*)p, mod_statusbar_statusbar_bindmap);
     
@@ -324,11 +324,13 @@ static void statusbar_do_update_natural_size(WStatusBar *p)
 }
 
 
-void statusbar_size_hints(WStatusBar *p, XSizeHints *h)
+void statusbar_size_hints(WStatusBar *p, WSizeHints *h)
 {
-    h->flags=PMaxSize|PMinSize;
+    h->min_set=TRUE;
     h->min_width=p->natural_w;
     h->min_height=p->natural_h;
+    
+    h->max_set=TRUE;
     h->max_width=p->natural_w;
     h->max_height=p->natural_h;
 }
@@ -456,42 +458,25 @@ static void statusbar_arrange_systray(WStatusBar *p)
 
 static void systray_adjust_size(WRegion *reg, WRectangle *g)
 {
-    XSizeHints hints;
-    
-    region_size_hints(reg, &hints);
-
     g->h=CF_STATUSBAR_SYSTRAY_HEIGHT;
     
-    xsizehints_correct(&hints, &g->w, &g->h, TRUE);
+    region_size_hints_correct(reg, &g->w, &g->h, TRUE);
 }
 
 
 
-static WRegion *statusbar_attach_simple(WStatusBar *sb,
-                                        WRegionAttachHandler *handler,
-                                        void *handlerparams)
+static WRegion *statusbar_do_attach_final(WStatusBar *sb,
+                                          WRegion *reg,
+                                          void *unused)
 {
-    WRegion *reg;
     WFitParams fp;
     WSBElem *el;
     
-    fp.g.x=0;
-    fp.g.y=0;
-    fp.mode=REGION_FIT_WHATEVER|REGION_FIT_BOUNDS;
-
-    reg=handler((WWindow*)sb, &fp, handlerparams);
-    
-    if(reg==NULL)
+    if(!ptrlist_insert_last(&sb->traywins, (Obj*)reg))
         return NULL;
     
-    if(!ptrlist_insert_last(&sb->traywins, (Obj*)reg)){
-        /* TODO: failure handling */
-        return NULL;
-    }
-    
     el=statusbar_associate_systray(sb, reg);
     if(el==NULL){
-        /* TODO: failure handling */
         ptrlist_remove(&sb->traywins, (Obj*)reg);
         return NULL;
     }
@@ -515,6 +500,30 @@ static WRegion *statusbar_attach_simple(WStatusBar *sb,
 }
 
 
+static bool statusbar_do_attach(WStatusBar *sb, WRegionAttachData *data)
+{
+    WFitParams fp;
+    
+    fp.g.x=0;
+    fp.g.y=0;
+    fp.g.h=CF_STATUSBAR_SYSTRAY_HEIGHT;
+    fp.g.w=CF_STATUSBAR_SYSTRAY_HEIGHT;
+    fp.mode=REGION_FIT_WHATEVER|REGION_FIT_BOUNDS;
+    
+    return (region_attach_helper((WRegion*)sb, (WWindow*)sb, &fp,
+                                 (WRegionDoAttachFn*)statusbar_do_attach_final, 
+                                 NULL, data)
+            !=NULL);
+}
+
+
+static bool statusbar_attach_ph(WStatusBar *sb, int flags,
+                                WRegionAttachData *data)
+{
+    return statusbar_do_attach(sb, data);
+}
+
+
 static WPHolder *statusbar_prepare_manage(WStatusBar *sb, 
                                           const WClientWin *cwin,
                                           const WManageParams *param,
@@ -524,8 +533,8 @@ static WPHolder *statusbar_prepare_manage(WStatusBar *sb,
         return NULL;
     
     return (WPHolder*)create_basicpholder((WRegion*)sb, 
-                                          ((WRegionDoAttachFnSimple*)
-                                           statusbar_attach_simple));
+                                          ((WBasicPHolderHandler*)
+                                           statusbar_attach_ph));
 }
 
 
@@ -534,10 +543,11 @@ static void statusbar_managed_remove(WStatusBar *sb, WRegion *reg)
     WSBElem *el;
         
     ptrlist_remove(&sb->traywins, (Obj*)reg);
-    region_unset_manager(reg, (WRegion*)sb);
     
     el=statusbar_unassociate_systray(sb, reg);
     
+    region_unset_manager(reg, (WRegion*)sb);
+
     if(el!=NULL && ioncore_g.opmode!=IONCORE_OPMODE_DEINIT){
         do_calc_systray_w(sb, el);
         statusbar_rearrange(sb, TRUE);
